@@ -28,9 +28,7 @@ from glue.ligolw import table
 from glue.ligolw import utils
 from glue.ligolw import ilwd
 from glue.ligolw.utils import process as ligolw_process
-from pylal.inspiral_metric import compute_metric
-from pylal.xlal.datatypes.real8frequencyseries import REAL8FrequencySeries
-from pylal.xlal.datatypes.snglinspiraltable import SnglInspiralTable
+from lal import REAL8FrequencySeries
 
 from optparse import OptionParser
 
@@ -38,7 +36,7 @@ from optparse import OptionParser
 from lalinspiral.sbank.bank import Bank
 from lalinspiral.sbank.tau0tau3 import proposals
 from lalinspiral.sbank.psds import noise_models, read_psd, get_PSD
-from lalinspiral.sbank.waveforms import waveforms
+from lalinspiral.sbank.waveforms import waveforms, SnglInspiralTable
 
 import lal
 
@@ -153,6 +151,8 @@ def parse_command_line():
     #
     parser.add_option("--approximant", choices=waveforms.keys(), metavar='|'.join(waveforms.keys()), default=None, help="Required. Specify the approximant to use for waveform generation.")
     parser.add_option("--use-metric", action="store_true", default=False, help="Use analytic approximation to the numerical match calculation (if available).")
+    parser.add_option("--duration-min", type=float, help="Set minimum allowed duration of the template waveform in seconds.")
+    parser.add_option("--duration-max", type=float, help="Set maximum allowed duration of the template waveform in seconds.")
 
     #
     # mass parameter options
@@ -185,6 +185,7 @@ def parse_command_line():
     #
     parser.add_option("--seed", help="Set the seed for the random number generator used by SBank for waveform parameter (masss, spins, ...) generation.", metavar="INT", default=1729, type="int")
     parser.add_option("--bank-seed", metavar="FILE[:APPROX]", help="Add templates from FILE to the initial bank. If APPROX is also specified, the templates from this seed bank will be computed with this approximant instead of the one specified by --approximant. Can be specified multiple times. Only the additional templates will be outputted.", action="append", default=[])
+    parser.add_option("--trial-waveforms", metavar="FILE", help="If supplied, instead of choosing points randomly, choose trial points from the sngl_inspiral table within the supplied XML file. Generation will terminate if the end of the file is reached unless any other termination condition is met first.")
 
     #
     # noise model options
@@ -198,7 +199,7 @@ def parse_command_line():
     parser.add_option("--flow", type="float", help="Required. Set the low-frequency cutoff to use for the match caluclation.")
     parser.add_option("--match-min",help="Set minimum match of the bank. Note that since this is a stochastic process, the requested minimal match may not be strictly guaranteed but should be fulfilled on a statistical basis. Default: 0.95.", type="float", default=0.95)
     parser.add_option("--convergence-threshold", metavar="N", help="Set the criterion for convergence of the stochastic bank. The code terminates when there are N rejected proposals for each accepted proposal, averaged over the last ten acceptances. Default 1000.", type="int", default=1000)
-    parser.add_option("--templates-max", metavar="N", help="Use this option to force the code to exit after generating a specified number N of templates. Note that the code may exit with fewer than N templates if the convergence criterion is met first.", type="int", default=float('inf'))
+    parser.add_option("--max-new-templates", metavar="N", help="Use this option to force the code to exit after accepting a specified number N of new templates. Note that the code may exit with fewer than N templates if the convergence criterion is met first.", type="int", default=float('inf'))
     parser.add_option("--cache-waveforms", default = False, action="store_true", help="A given waveform in the template bank will be used many times throughout the bank generation process. You can save a considerable amount of CPU by caching the waveform from the first time it is generated; however, do so only if you are sure that storing the waveforms in memory will not overload the system memory.")
     parser.add_option("--coarse-match-df", type="float", default=None, help="If given, use this value of df to quickly test if the mismatch is less than 4 times the minimal mismatch. This can quickly reject points at high values of df, that will not have high overlaps at smaller df values. This can be used to speed up the sbank process.")
     parser.add_option("--iterative-match-df-max", type="float", default=None, help="If this option is given it will enable sbank using larger df values than 1 / data length when computing overlaps. Sbank will then compute a match at this value, and at half this value, if the two values agree to 0.1% the value obtained will be taken as the actual value. If the values disagree the match will be computed again using a df another factor of 2 smaller until convergence or a df of 1/ data_length, is reached.")
@@ -214,6 +215,7 @@ def parse_command_line():
     parser.add_option("--instrument", metavar="IFO", help="Specify the instrument for which to generate a template bank. This option is used for naming of the output file but also for reading in PSDs or template bank seeds from file.")
     parser.add_option("--gps-start-time", type="int", default=0, help="GPS time of start. Used only for naming of output file.", metavar="INT")
     parser.add_option("--gps-end-time", type="int", default=999999999, help="GPS time of end. Used only for naming of output file", metavar="INT")
+    parser.add_option("--output-filename", default=None, help="Over-ride default filenaming and use this as the output filename.")
     parser.add_option("--user-tag", default=None, help="Apply descriptive tag to output filename.")
     parser.add_option("--verbose", default=False,action="store_true", help="Be verbose and write diagnostic information out to file.")
 
@@ -261,7 +263,7 @@ def parse_command_line():
             parser.error("unphysical spin2 bounds: [%.2f, %.2f]" % (opts.spin2_min, opts.spin2_max))
     elif all_nsbh_spin_opts:
         if not -1 <= opts.bh_spin_min <= opts.bh_spin_max <= 1:
-            parser.error("unphysical BH spin bounds: [%.2f, %.2f]" % (opts.bh_spin_min, opts.bs_spin_max))
+            parser.error("unphysical BH spin bounds: [%.2f, %.2f]" % (opts.bh_spin_min, opts.bh_spin_max))
         if not -1 <= opts.ns_spin_min <= opts.ns_spin_max <= 1:
             parser.error("unphysical NS spin bounds: [%.2f, %.2f]" % (opts.ns_spin_min, opts.ns_spin_max))
     else:
@@ -285,18 +287,19 @@ def parse_command_line():
     if (opts.mchirp_boundaries_file is not None) ^ (opts.mchirp_boundaries_index is not None):
         parser.error("must supply both --mchirp-boundaries-file and --mchirp-boundaries-index or neither")
 
-    if opts.mchirp_boundaries_file and (opts.mchirp_min or opts.mchirp_max):
-        parser.error("--mchirp-boundaries-file supercedes --mchirp-min and --mchirp-max")
-
     if opts.mchirp_boundaries_file:
         boundaries = [float(line) for line in open(opts.mchirp_boundaries_file)]
         if opts.mchirp_boundaries_index > len(boundaries):
             raise ValueError("mchirp boundaries file not long enough for requested index")
 
         if opts.mchirp_boundaries_index > 0:
-            opts.mchirp_min = float(boundaries[opts.mchirp_boundaries_index - 1])
+            boundary_mchirp_min = float(boundaries[opts.mchirp_boundaries_index - 1])
+            if opts.mchirp_min is None or opts.mchirp_min < boundary_mchirp_min:
+                opts.mchirp_min = boundary_mchirp_min
         if opts.mchirp_boundaries_index + 1 < len(boundaries):
-            opts.mchirp_max = float(boundaries[opts.mchirp_boundaries_index])
+            boundary_mchirp_max = float(boundaries[opts.mchirp_boundaries_index])
+            if opts.mchirp_max is None or opts.mchirp_max > boundary_mchirp_max:
+                opts.mchirp_max = boundary_mchirp_max
 
     return opts, args
 
@@ -309,7 +312,9 @@ opts, args = parse_command_line()
 #
 # determine output bank filename
 #
-if opts.user_tag:
+if opts.output_filename:
+    fout = opts.output_filename
+elif opts.user_tag:
     fout = "%s-SBANK_%s-%d-%d.xml.gz" % (opts.instrument, opts.user_tag, opts.gps_start_time, opts.gps_end_time-opts.gps_start_time)
 else:
     fout = "%s-SBANK-%d-%d.xml.gz" % (opts.instrument, opts.gps_start_time, opts.gps_end_time-opts.gps_start_time)
@@ -317,14 +322,14 @@ else:
 #
 # choose waveform approximant
 #
-waveform = waveforms[opts.approximant]
+tmplt_class = waveforms[opts.approximant]
 
 #
 # choose noise model
 #
 if opts.reference_psd is not None:
     psd = read_psd(opts.reference_psd)[opts.instrument]
-    f_orig = psd.f0 + np.arange(len(psd.data)) * psd.deltaF
+    f_orig = psd.f0 + np.arange(len(psd.data.data)) * psd.deltaF
     f_max_orig = max(f_orig)
     if opts.fhigh_max:
         if opts.fhigh_max > f_max_orig:
@@ -336,7 +341,7 @@ if opts.reference_psd is not None:
                 % f_max_orig
         opts.fhigh_max = float(f_max_orig)
 
-    interpolator = UnivariateSpline(f_orig, np.log(psd.data), s=0)
+    interpolator = UnivariateSpline(f_orig, np.log(psd.data.data), s=0)
 
     # spline extrapolation may lead to unexpected results,
     # so set the PSD to infinity above the max original frequency
@@ -344,11 +349,6 @@ if opts.reference_psd is not None:
 else:
     noise_model = noise_models[opts.noise_model]
 
-# Set up PSD for metric computation
-# calling into pylal, so need pylal types
-psd = REAL8FrequencySeries(name="psd", f0=0., deltaF=1., data=get_PSD(1., opts.flow, 1570., noise_model))
-
-
 #
 # initialize the bank
 #
@@ -366,7 +366,7 @@ for file_approx in opts.bank_seed:
 
     # add templates to bank
     tmpdoc = utils.load_filename(seed_file, contenthandler=ContentHandler)
-    sngl_inspiral = table.get_table(tmpdoc, lsctables.SnglInspiralTable.tableName)
+    sngl_inspiral = lsctables.SnglInspiralTable.get_table(tmpdoc)
     seed_waveform = waveforms[approx]
     bank.add_from_sngls(sngl_inspiral, seed_waveform)
 
@@ -386,8 +386,8 @@ if opts.verbose:
 if opts.checkpoint and os.path.exists( fout + "_checkpoint.gz" ):
 
     xmldoc = utils.load_filename(fout + "_checkpoint.gz", contenthandler=ContentHandler)
-    tbl = table.get_table(xmldoc, lsctables.SnglInspiralTable.tableName)
-    [bank.insort(t) for t in Bank.from_sngls(tbl, waveform, noise_model, opts.flow, opts.use_metric, opts.cache_waveforms, opts.neighborhood_size, opts.neighborhood_param, coarse_match_df=opts.coarse_match_df, iterative_match_df_max=opts.iterative_match_df_max, fhigh_max=opts.fhigh_max)]
+    tbl = lsctables.SnglInspiralTable.get_table(xmldoc)
+    [bank.insort(t) for t in Bank.from_sngls(tbl, tmplt_class, noise_model, opts.flow, opts.use_metric, opts.cache_waveforms, opts.neighborhood_size, opts.neighborhood_param, coarse_match_df=opts.coarse_match_df, iterative_match_df_max=opts.iterative_match_df_max, fhigh_max=opts.fhigh_max)]
 
     if opts.verbose:
         print >>sys.stdout,"Found checkpoint file %s with %d precomputed templates." % (fout + "_checkpoint.gz", len(tbl))
@@ -427,22 +427,31 @@ process = ligolw_process.register_to_xmldoc(xmldoc, "lalapps_cbc_sbank",
 # populate params dictionary to be passed to the generators
 #
 
-params = {'mass1': (opts.mass1_min, opts.mass1_max),
+if opts.trial_waveforms:
+    trialdoc = utils.load_filename(opts.trial_waveforms, contenthandler=ContentHandler, gz=opts.trial_waveforms.endswith('.gz'))
+    trial_sngls = lsctables.SnglInspiralTable.get_table(trialdoc)
+    proposal = [tmplt_class.from_sngl(t, bank=bank) for t in trial_sngls]
+
+else:
+    params = {'mass1': (opts.mass1_min, opts.mass1_max),
           'mass2': (opts.mass2_min, opts.mass2_max),
           'mtotal': (opts.mtotal_min, opts.mtotal_max),
           'mratio': (opts.qmin, opts.qmax),
           'mchirp': (opts.mchirp_min, opts.mchirp_max)}
 
-if opts.ns_bh_boundary_mass is not None:
-    params['ns_bh_boundary_mass'] = opts.ns_bh_boundary_mass
-    params['bh_spin'] = (opts.bh_spin_min, opts.bh_spin_max)
-    params['ns_spin'] = (opts.ns_spin_min, opts.ns_spin_max)
-else:
-    params['spin1'] = (opts.spin1_min, opts.spin1_max)
-    params['spin2'] = (opts.spin2_min, opts.spin2_max)
+    if opts.ns_bh_boundary_mass is not None:
+        params['ns_bh_boundary_mass'] = opts.ns_bh_boundary_mass
+        params['bh_spin'] = (opts.bh_spin_min, opts.bh_spin_max)
+        params['ns_spin'] = (opts.ns_spin_min, opts.ns_spin_max)
+    else:
+        params['spin1'] = (opts.spin1_min, opts.spin1_max)
+        params['spin2'] = (opts.spin2_min, opts.spin2_max)
+    if opts.duration_min is not None or opts.duration_max is not None:
+        params['duration'] = (opts.duration_min, opts.duration_max)
 
-# get the correct generator for the chosen approximant
-proposal = proposals[opts.approximant](opts.flow, **params)
+    # get the correct generator for the chosen approximant
+    proposal = proposals[opts.approximant](opts.flow, tmplt_class, bank,
+                                           **params)
 
 
 # For robust convergence, ensure that an average of kmax/len(ks) of
@@ -450,11 +459,27 @@ proposal = proposals[opts.approximant](opts.flow, **params)
 ks = deque(10*[1], maxlen=10)
 k = 0 # k is nprop per iteration
 nprop = 1  # count total number of proposed templates
-status_format = "\t".join("%s: %s" % name_format for name_format in zip(waveform.param_names, waveform.param_formats))
-while ((k + float(sum(ks)))/len(ks) < opts.convergence_threshold) and len(bank) < opts.templates_max:
-    tmplt = waveform(*proposal.next(), bank=bank)
-    k += 1
-    nprop += 1
+status_format = "\t".join("%s: %s" % name_format for name_format in zip(tmplt_class.param_names, tmplt_class.param_formats))
+
+#
+# main working loop
+#
+for tmplt in proposal:
+
+    #
+    # check if stopping criterion has been reached
+    #
+    if not (((k + float(sum(ks)))/len(ks) < opts.convergence_threshold) and \
+            (len(tbl) < opts.max_new_templates)):
+        break
+
+    # accounting for number of proposals
+    k += 1 # since last acceptance
+    nprop += 1 # total throughout lifetime of process
+
+    #
+    # check if proposal is already covered by existing templates
+    #
     match, matcher = bank.covers(tmplt, opts.match_min)
     if match < opts.match_min:
         bank.insort(tmplt)
@@ -472,11 +497,14 @@ while ((k + float(sum(ks)))/len(ks) < opts.convergence_threshold) and len(bank)
             row = tmplt.to_sngl()
             # Event ids must be unique, or the table isn't valid, SQL needs this
             row.event_id = ilwd.ilwdchar('sngl_inspiral:event_id:%d' %(len(bank),))
+            # If we figure out how to use metaio's SnglInspiralTable the
+            # following change then defines the event_id
+            #curr_id = EventIDColumn()
+            #curr_id.id = len(bank)
+            #curr_id.snglInspiralTable = row
+            #row.event_id = curr_id
             row.ifo = opts.instrument
             row.process_id = process.process_id
-            row.Gamma0, row.Gamma1, row.Gamma2, row.Gamma3, row.Gamma4, row.Gamma5,\
-                row.Gamma6, row.Gamma7, row.Gamma8, row.Gamma9 = \
-                compute_metric(opts.flow, 1570., 4, row.tau0, row.tau3, psd)
             tbl.append(row)
 
         if opts.checkpoint and not len(bank) % opts.checkpoint:
