@@ -35,21 +35,21 @@ extern "C" {
 
 /* VCS information defines */
 #define LALAPPS_NAME "LALApps"
-#define LALAPPS_VCS_ID "5e0d77a3a68496f08b41b6e21a608a25dca063da"
-#define LALAPPS_VCS_DATE "2016-06-26 21:26:32 +0000"
+#define LALAPPS_VCS_ID "6c73f0c9d565841e343bedb55f412531b52ec2ac"
+#define LALAPPS_VCS_DATE "2016-10-10 17:31:35 +0000"
 #define LALAPPS_VCS_BRANCH "None"
-#define LALAPPS_VCS_TAG "lalapps-v6.19.0"
+#define LALAPPS_VCS_TAG "lalapps-v6.20.0"
 #define LALAPPS_VCS_AUTHOR "Adam Mercer <adam.mercer@ligo.org>"
-#define LALAPPS_VCS_COMMITTER "Adam Mercer <ramercer@gmail.com>"
+#define LALAPPS_VCS_COMMITTER "Adam Mercer <adam.mercer@ligo.org>"
 #define LALAPPS_VCS_STATUS "CLEAN: All modifications committed"
 
 /* Identable VCS information defines */
-#define LALAPPS_VCS_IDENT_ID "$" "LALAppsId: 5e0d77a3a68496f08b41b6e21a608a25dca063da " "$"
-#define LALAPPS_VCS_IDENT_DATE "$" "LALAppsDate: 2016-06-26 21:26:32 +0000 " "$"
+#define LALAPPS_VCS_IDENT_ID "$" "LALAppsId: 6c73f0c9d565841e343bedb55f412531b52ec2ac " "$"
+#define LALAPPS_VCS_IDENT_DATE "$" "LALAppsDate: 2016-10-10 17:31:35 +0000 " "$"
 #define LALAPPS_VCS_IDENT_BRANCH "$" "LALAppsBranch: None " "$"
-#define LALAPPS_VCS_IDENT_TAG "$" "LALAppsTag: lalapps-v6.19.0 " "$"
+#define LALAPPS_VCS_IDENT_TAG "$" "LALAppsTag: lalapps-v6.20.0 " "$"
 #define LALAPPS_VCS_IDENT_AUTHOR "$" "LALAppsAuthor: Adam Mercer <adam.mercer@ligo.org> " "$"
-#define LALAPPS_VCS_IDENT_COMMITTER "$" "LALAppsCommitter: Adam Mercer <ramercer@gmail.com> " "$"
+#define LALAPPS_VCS_IDENT_COMMITTER "$" "LALAppsCommitter: Adam Mercer <adam.mercer@ligo.org> " "$"
 #define LALAPPS_VCS_IDENT_STATUS "$" "LALAppsStatus: CLEAN: All modifications committed " "$"
 
 /* Header VCS information structure */
