@@ -82,6 +82,7 @@ typedef struct{
   REAL8   spacingT;           /**< spacing in time of periapse passage*/
   REAL8   spacingP;           /**< spacing in period*/
   INT4    numCand;            /**< number of candidates to keep in output toplist */
+  CHAR    *linesToCleanFilenames; /**< comma-separated list of filenames with known lines for each ifo */
   CHAR    *pairListInputFilename;  /**< input filename containing list of sft index pairs (if not provided, determine list of pairs */
   CHAR    *pairListOutputFilename; /**< output filename to write list of sft index pairs */
   CHAR    *sftListOutputFilename;  /**< output filename to write list of sfts */
@@ -97,19 +98,22 @@ typedef struct{
 typedef struct{
   SFTCatalog *catalog; /**< catalog of SFTs */
   EphemerisData *edat; /**< ephemeris data */
+  LALStringVector *lineFiles; /**< list of line files */
   REAL8   refTime;     /**< reference time for pulsar phase definition */
 } ConfigVariables;
 
 #define TRUE (1==1)
 #define FALSE (1==0)
 #define MAXFILENAMELENGTH 512
+#define MAXLINELENGTH 1024
 
 /* local function prototypes */
 int XLALInitUserVars ( UserInput_t *uvar );
 int XLALInitializeConfigVars (ConfigVariables *config, const UserInput_t *uvar);
 int XLALDestroyConfigVars (ConfigVariables *config);
 int GetNextCrossCorrTemplate(BOOLEAN *binaryParamsFlag, BOOLEAN *firstPoint, PulsarDopplerParams *dopplerpos, PulsarDopplerParams *binaryTemplateSpacings, PulsarDopplerParams *minBinaryTemplate, PulsarDopplerParams *maxBinaryTemplate, UINT8 *fCount, UINT8 *aCount, UINT8 *tCount, UINT8 *pCount, UINT8 fSpacingNum, UINT8 aSpacingNum, UINT8 tSpacingNum, UINT8 pSpacingNum);
-
+UINT4 pcc_count_csv( CHAR *csvline );
+INT4 XLALFindBadBins ( UINT4Vector *badBinData, INT4 binCount, REAL8 flo, REAL8 fhi, REAL8 f0, REAL8 deltaF, UINT4 length) ;
 /** @} */
 
 int main(int argc, char *argv[]){
@@ -428,6 +432,158 @@ int main(int argc, char *argv[]){
 
     }
 
+  /* Parse the list of lines to avoid (if given) */
+
+  MultiUINT4Vector *badBins = NULL;
+
+#define PCC_LINEFILE_HEADER "%% %2s lines cleaning file for O1\n"\
+"%%\n"\
+"%% File contains %d (non-comment) lines\n"\
+"%%\n"\
+"%% Column 1 - frequency spacing (Hz) of comb (or frequency of single line)\n"\
+"%% Column 2 - comb type (0 - singlet, 1 - comb with fixed width, 2 - comb with scaling width)\n"\
+"%% Column 3 - frequency offset of 1st visible harmonic (Hz)\n"\
+"%% Column 4 - index of first visible harmonic\n"\
+"%% Column 5 - index of last visible harmonic\n"\
+"%% Column 6 - width of left band (Hz)\n"\
+"%% Column 7 - width of right band (Hz)\n"\
+"%%\n"\
+"%% For fixed-width combs, veto the band:\n"\
+"%%     [offset+index*spacing-leftwidth, offset+index*spacing+rightwidth]\n"\
+"%% For scaling-width combs, veto the band:\n"\
+"%%     [offset+index*spacing-index*leftwidth, offset+index*spacing+index*rightwidth]\n"\
+"%%"
+#define PCC_LINEFILE_BODY "%lf %d %lf %d %d %lf %lf"
+
+  if ( config.lineFiles != NULL) {
+    if((badBins = XLALCalloc(1, sizeof(badBins))) == NULL){
+      XLAL_ERROR(XLAL_ENOMEM);
+    }
+    UINT4 numDets = inputSFTs->length;
+    badBins->length = numDets;
+    if((badBins->data = XLALCalloc(numDets, sizeof(*badBins->data))) == NULL){
+      XLAL_ERROR(XLAL_ENOMEM);
+    }
+
+    for ( UINT4 i_f=0 ; i_f < config.lineFiles->length ; i_f++ ) {
+      /* printf("i_f=%d\n",i_f); */
+      if((fp = fopen(config.lineFiles->data[i_f], "r")) == NULL){
+	LogPrintf ( LOG_CRITICAL, "%s: didn't find line file with name %s\n", __func__, config.lineFiles->data[i_f]);
+	XLAL_ERROR( XLAL_EINVAL );
+      }
+      CHAR ifo[3];
+      UINT4 numLines;
+      if(fscanf(fp,PCC_LINEFILE_HEADER,&ifo[0],&numLines)==EOF){
+	LogPrintf ( LOG_CRITICAL, "can't parse header of line file %s\n",config.lineFiles->data[i_f]);
+	XLAL_ERROR( XLAL_EINVAL );
+      }
+      fclose(fp);
+
+      UINT4 detInd = 0;
+      for ( ; detInd < numDets ; detInd++ ){
+	if ( strcmp(ifo,inputSFTs->data[detInd]->data[0].name) == 0 ) break;
+      } /*find the detctor index of the ifo and break the loop*/
+      if (detInd >= numDets){
+	LogPrintf ( LOG_CRITICAL, "%s: didn't find index for IFO %s\n", __func__, ifo);
+	XLAL_ERROR( XLAL_EINVAL );
+      }
+      UINT4 numSFTFreqs = inputSFTs->data[detInd]->data[0].data->length;
+      REAL8 f0 = inputSFTs->data[detInd]->data[0].f0;
+      if ( deltaF != inputSFTs->data[detInd]->data[0].deltaF ){
+	LogPrintf ( LOG_CRITICAL, "%s: deltaF = %f disagrees with SFT deltaF = %f", __func__, deltaF, inputSFTs->data[detInd]->data[0].deltaF );
+	XLAL_ERROR( XLAL_EINVAL );
+      }
+
+      if ((badBins->data[detInd] = XLALCreateUINT4Vector ( numSFTFreqs ) ) == NULL){
+	LogPrintf ( LOG_CRITICAL, "%s: XLALCreateUINT4Vector() failed with errno=%d\n", __func__, xlalErrno );
+	XLAL_ERROR( XLAL_EFUNC );
+      }
+      /* printf("%d\n",badBins->data[detInd]->length); */
+      INT4 binCount = 0;
+
+      if((fp = fopen(config.lineFiles->data[i_f], "r")) == NULL){
+	LogPrintf ( LOG_CRITICAL, "%s: didn't find line file with name %s\n", __func__, config.lineFiles->data[i_f]);
+	XLAL_ERROR( XLAL_EINVAL );
+      }
+      CHAR thisline[MAXLINELENGTH];
+      UINT4 linesRead = 0;
+      while (fgets(thisline, sizeof thisline, fp)) {
+	if ( thisline[0] == '%' ) continue;
+	REAL8 spacing;
+	UINT4 combtype;
+	REAL8 offset;
+	UINT4 firstindex, lastindex;
+	REAL8 leftwidth, rightwidth;
+	UINT4 numRead = sscanf(thisline,PCC_LINEFILE_BODY, &spacing, &combtype, &offset, &firstindex, &lastindex, &leftwidth, &rightwidth);
+	if(numRead!=7){
+	  LogPrintf ( LOG_CRITICAL, "Failed to read data out of line file %s: needed 7, got %d\n", config.lineFiles->data[i_f], numRead );
+	  XLAL_ERROR( XLAL_EINVAL );
+	}
+	/*	printf(PCC_LINEFILE_BODY "\n", spacing, combtype, offset, firstindex, lastindex, leftwidth, rightwidth); */
+	switch (combtype){
+	case 0: /* singlet line */
+	  if ( firstindex != lastindex ){
+	    LogPrintf ( LOG_CRITICAL, "Error in file %s: singlet line with firstindex=%d,lastindex=%d\n", config.lineFiles->data[i_f], firstindex, lastindex );
+	    XLAL_ERROR( XLAL_EINVAL );
+	  }
+	  binCount = XLALFindBadBins( badBins->data[detInd], binCount, offset+firstindex*spacing-leftwidth, offset+lastindex*spacing+rightwidth, f0, deltaF, numSFTFreqs);
+	  if (binCount < 0){
+	    LogPrintf ( LOG_CRITICAL, "%s: XLALFindBadBins() failed with errno=%d\n", __func__, xlalErrno );
+	    XLAL_ERROR( XLAL_EFUNC );
+	  }
+	  /* printf ("veto %f to %f\n", offset+firstindex*spacing-leftwidth, offset+lastindex*spacing+rightwidth); */
+	  break;
+	case 1: /* fixed-width comb */
+	  if ( firstindex > lastindex ){
+	    LogPrintf ( LOG_CRITICAL, "Error in file %s: comb with firstindex=%d,lastindex=%d\n", config.lineFiles->data[i_f], firstindex, lastindex );
+	    XLAL_ERROR( XLAL_EINVAL );
+	  }
+	  for ( UINT4 index0 = firstindex ; index0 <= lastindex; index0++ ) {
+	    binCount = XLALFindBadBins( badBins->data[detInd], binCount, offset+index0*spacing-leftwidth, offset+index0*spacing+rightwidth, f0, deltaF, numSFTFreqs);
+	    if (binCount < 0){
+	      LogPrintf ( LOG_CRITICAL, "%s: XLALFindBadBins() failed with errno=%d\n", __func__, xlalErrno );
+	      XLAL_ERROR( XLAL_EFUNC );
+	    }
+	    /* printf ("veto %f to %f\n", offset+index0*spacing-leftwidth, offset+index0*spacing+rightwidth); */
+	  }
+	  break;
+	case 2: /* scaling-width comb */
+	  if ( firstindex > lastindex ){
+	    LogPrintf ( LOG_CRITICAL, "Error in file %s: comb with firstindex=%d,lastindex=%d\n", config.lineFiles->data[i_f], firstindex, lastindex );
+	    XLAL_ERROR( XLAL_EINVAL );
+	  }
+	  for ( UINT4 index0 = firstindex ; index0 <= lastindex; index0++ ) {
+	    binCount = XLALFindBadBins( badBins->data[detInd], binCount, offset+index0*spacing-index0*leftwidth, offset+index0*spacing+index0*rightwidth, f0, deltaF, numSFTFreqs);
+	    if (binCount < 0){
+	      LogPrintf ( LOG_CRITICAL, "%s: XLALFindBadBins() failed with errno=%d\n", __func__, xlalErrno );
+	      XLAL_ERROR( XLAL_EFUNC );
+	    }
+	    /* printf ("veto %f to %f\n", offset+index0*spacing-index0*leftwidth, offset+index0*spacing+index0*rightwidth); */
+	  }
+	  break;
+	default:
+	  LogPrintf ( LOG_CRITICAL, "Unrecognized combtype %d\n", combtype );
+	  XLAL_ERROR( XLAL_EINVAL );
+	}
+
+	linesRead++;
+      } /* while (fgets(thisline, sizeof thisline, fp)) */
+      if (linesRead != numLines){
+	LogPrintf ( LOG_CRITICAL, "Read %d lines out of %s but expected %d", linesRead, config.lineFiles->data[i_f], numLines );
+	XLAL_ERROR( XLAL_EFUNC );
+      }
+      if ( binCount == 0 ){
+	XLALDestroyUINT4Vector( badBins->data[detInd] );
+	badBins->data[detInd] = NULL;
+      } else {
+	if ( (badBins->data[detInd]->data = XLALRealloc(badBins->data[detInd]->data, binCount*sizeof(UINT4)) ) == NULL){
+	  XLAL_ERROR(XLAL_ENOMEM);
+	}
+	badBins->data[detInd]->length = binCount;
+      }
+    } /* for ( UINT4 i_f=0 ; i_f < config.lineFiles->length ; i_f++ ) */
+  } /* if ( config->lineFiles ) */
+
   /* Get weighting factors for calculation of metric */
   /* note that the sigma-squared is now absorbed into the curly G
      because the AM coefficients are noise-weighted. */
@@ -497,9 +653,10 @@ int main(int argc, char *argv[]){
   REAL8 ccStat = 0;
   REAL8 evSquared = 0;
   REAL8 estSens = 0; /*estimated sensitivity(4.13)*/
+  REAL8 weightedMuTAve = 0;
 
   /*Get metric diagonal components, also estimate sensitivity i.e. E[rho]/(h0)^2 (4.13)*/
-  if ( (XLALCalculateLMXBCrossCorrDiagMetric(&estSens, &old_diagff, &old_diagaa, &old_diagTT, &old_diagpp, thisBinaryTemplate, GammaAve, sftPairs, sftIndices, inputSFTs, multiWeights /*, kappaValues*/)  != XLAL_SUCCESS ) ) {
+  if ( (XLALCalculateLMXBCrossCorrDiagMetric(&estSens, &old_diagff, &old_diagaa, &old_diagTT, &old_diagpp, &weightedMuTAve, thisBinaryTemplate, GammaAve, sftPairs, sftIndices, inputSFTs, multiWeights /*, kappaValues*/)  != XLAL_SUCCESS ) ) {
     LogPrintf ( LOG_CRITICAL, "%s: XLALCalculateLMXBCrossCorrDiagMetric() failed with errno=%d\n", __func__, xlalErrno );
     XLAL_ERROR( XLAL_EFUNC );
   }
@@ -667,7 +824,7 @@ int main(int argc, char *argv[]){
 	  }
 	}
 
-      if ( (XLALGetDopplerShiftedFrequencyInfo( shiftedFreqs, lowestBins, expSignalPhases, sincList, uvar.numBins, &dopplerpos, sftIndices, inputSFTs, multiBinaryTimes, Tsft )  != XLAL_SUCCESS ) ) {
+      if ( (XLALGetDopplerShiftedFrequencyInfo( shiftedFreqs, lowestBins, expSignalPhases, sincList, uvar.numBins, &dopplerpos, sftIndices, inputSFTs, multiBinaryTimes, badBins, Tsft )  != XLAL_SUCCESS ) ) {
 	LogPrintf ( LOG_CRITICAL, "%s: XLALGetDopplerShiftedFrequencyInfo() failed with errno=%d\n", __func__, xlalErrno );
 	XLAL_ERROR( XLAL_EFUNC );
       }
@@ -749,8 +906,9 @@ int main(int argc, char *argv[]){
     fprintf(fp, "TemplatenumTotal = %" LAL_UINT8_FORMAT "\n",(fSpacingNum + 1) * (aSpacingNum + 1) * (tSpacingNum + 1) * (pSpacingNum + 1));
     fprintf(fp, "Sens = %.9g\n", estSens); /*(E[rho]/h0^2)^2*/
     fprintf(fp, "h0_min_SNR10 = %.9g\n", h0Sens); /*for rho = 10 in our pipeline*/
-    fprintf(fp, "startTime = %" LAL_INT4_FORMAT "\n", computingStartGPSTime.gpsSeconds); /*start time in GPS-time*/
-    fprintf(fp, "endTime = %" LAL_INT4_FORMAT "\n", computingEndGPSTime.gpsSeconds); /*end time in GPS-time*/
+    fprintf(fp, "weightedMutAve = %.9f\n", weightedMuTAve); /*weighted average of mean SFT from each pair of SFT*/
+    fprintf(fp, "jobStartTime = %" LAL_INT4_FORMAT "\n", computingStartGPSTime.gpsSeconds); /*job start time in GPS-time*/
+    fprintf(fp, "jobEndTime = %" LAL_INT4_FORMAT "\n", computingEndGPSTime.gpsSeconds); /*job end time in GPS-time*/
     fprintf(fp, "computingTime = %" LAL_UINT4_FORMAT "\n", computingTime); /*total time in sec*/
     fprintf(fp, "SFTnum = %" LAL_UINT4_FORMAT "\n", sftIndices->length); /*total number of SFT*/
     fprintf(fp, "pairnum = %" LAL_UINT4_FORMAT "\n", sftPairs->length); /*total number of pair of SFT*/
@@ -761,6 +919,8 @@ int main(int argc, char *argv[]){
     XLALFree(CMDInputStr);
   }
 
+  /* FIXME: Need to destroy badBins */
+
   XLALFree(VCSInfoString);
   XLALDestroyCOMPLEX8Vector ( expSignalPhases );
   XLALDestroyUINT4Vector ( lowestBins );
@@ -873,6 +1033,7 @@ int XLALInitUserVars (UserInput_t *uvar)
   XLALRegisterUvarMember( spacingT,       REAL8, 0,  OPTIONAL, "Desired periapse passage time spacing");
   XLALRegisterUvarMember( spacingP,       REAL8, 0,  OPTIONAL, "Desired period spacing");
   XLALRegisterUvarMember( numCand,         INT4, 0,  OPTIONAL, "Number of candidates to keep in toplist");
+  XLALRegisterUvarMember( linesToCleanFilenames, STRING, 0,  OPTIONAL, "Comma-separated list of line files");
   XLALRegisterUvarMember( pairListInputFilename, STRING, 0,  OPTIONAL, "Name of file from which to read list of SFT pairs");
   XLALRegisterUvarMember( pairListOutputFilename, STRING, 0,  OPTIONAL, "Name of file to which to write list of SFT pairs");
   XLALRegisterUvarMember( sftListOutputFilename, STRING, 0,  OPTIONAL, "Name of file to which to write list of SFTs (for sanity checks)");
@@ -929,9 +1090,30 @@ int XLALInitializeConfigVars (ConfigVariables *config, const UserInput_t *uvar)
     XLAL_ERROR( XLAL_EFUNC );
   }
 
-  /* initialize ephemeris data*/
+  /* initialize ephemeris data */
   XLAL_CHECK ( (config->edat = XLALInitBarycenter ( uvar->ephemEarth, uvar->ephemSun )) != NULL, XLAL_EFUNC );
 
+  /* parse comma-separated list of lines files */
+  config->lineFiles = NULL;
+
+  if (XLALUserVarWasSet(&uvar->linesToCleanFilenames))
+    {
+      CHAR *tmpstring = NULL;
+      XLAL_CHECK ( (tmpstring = XLALStringDuplicate( uvar->linesToCleanFilenames )) != NULL, XLAL_EFUNC );
+
+      UINT4 numfiles = pcc_count_csv( tmpstring );
+
+      LALFree( tmpstring );
+      XLAL_CHECK ( (tmpstring = XLALStringDuplicate( uvar->linesToCleanFilenames )) != NULL, XLAL_EFUNC );
+
+      for ( UINT4 i = 0 ; i < numfiles ; i++ ){
+	CHAR *pcc_tmpfile = NULL;
+	XLAL_CHECK ( (pcc_tmpfile = XLALStringToken( &tmpstring, ",", 0))!= NULL, XLAL_EFUNC );
+	XLAL_CHECK ( (config->lineFiles = XLALAppendString2Vector( config->lineFiles, pcc_tmpfile ))!= NULL, XLAL_EFUNC );
+
+      }
+    }
+
   return XLAL_SUCCESS;
 
 }
@@ -942,6 +1124,7 @@ int XLALDestroyConfigVars (ConfigVariables *config)
 {
   XLALDestroySFTCatalog(config->catalog);
   XLALDestroyEphemerisData(config->edat);
+  XLALDestroyStringVector(config->lineFiles);
   return XLAL_SUCCESS;
 }
 /* XLALDestroyConfigVars() */
@@ -1034,3 +1217,88 @@ int GetNextCrossCorrTemplate(BOOLEAN *binaryParamsFlag, BOOLEAN *firstPoint, Pul
     }
 }
 
+/* Copied from ppe_utils.c by Matt Pitkin */
+
+/**
+ * \brief Counts the number of comma separated values in a string
+ *
+ * This function counts the number of comma separated values in a given input string.
+ *
+ * \param csvline [in] Any string
+ *
+ * \return The number of comma separated value in the input string
+ */
+UINT4 pcc_count_csv( CHAR *csvline ){
+  CHAR *inputstr = NULL;
+  UINT4 count = 0;
+
+  inputstr = XLALStringDuplicate( csvline );
+
+  /* count number of commas */
+  while(1){
+    if( XLALStringToken(&inputstr, ",", 0) == NULL ){ XLAL_ERROR( XLAL_EFUNC, "Error... problem counting number of commas!" ); }
+
+    if ( inputstr == NULL ) { break; }
+
+    count++;
+  }
+
+  return count+1;
+}
+
+/** Convert a range of contaminated frequencies into a set of bins to zero out */
+/* Returns the running number of zeroed bins */
+INT4 XLALFindBadBins
+  (
+   UINT4Vector *badBinData, /* Modified: running list of bad bins */
+   INT4 binCount, /* Input: number of bins already in list */
+   REAL8 flo, /* Input: Lower end of contaminated frequency range */
+   REAL8 fhi, /* Input: Upper end of contaminated frequency range */
+   REAL8 f0, /* Input: Base frequency of frequency series */
+   REAL8 deltaF, /* Input: Frequency step of frequency series */
+   UINT4 length /* Input: Size of frequency series */
+   ) 
+{
+
+  /* printf ("veto %f to %f\n", flo, fhi); */
+
+  /* printf ("Series has %d bins starting at %f with %f spacing\n", length, f0, deltaF); */
+  /* printf ("Last bin is %f\n", f0 + length * deltaF); */
+
+  INT4 newBinCount = binCount;
+  INT4 firstBadBin = (INT4) floor ( ( flo - f0 ) / deltaF ); /*use floor to get the lowest contaminated bin*/
+  /* printf ("firstBadBin = %d\n",firstBadBin); */
+  if ( firstBadBin < 0 ) firstBadBin = 0;
+  INT4 lastBadBin = (INT4) ceil ( ( fhi - f0 ) / deltaF ); /*use ceil to get the highest contaminated bin make sure to extend the boundary*/
+  /* printf ("lastBadBin = %d\n",lastBadBin); */
+  if ( lastBadBin >= (INT4) length ) lastBadBin = (INT4) (length-1);
+
+  /* printf ("%d %d\n", firstBadBin, lastBadBin); */
+
+  for ( INT4 badBin = firstBadBin ; badBin <= lastBadBin ; badBin++ ){
+    /* printf ("%d %d %d\n", badBin, firstBadBin, lastBadBin); */
+    if (newBinCount > (INT4) length) {
+      LogPrintf ( LOG_CRITICAL, "%s: requested bin %d longer than series length %d\n", __func__, newBinCount, length);
+      XLAL_ERROR( XLAL_EINVAL );
+    }
+    UINT4 checkIfBinAppeared = 0;
+    for (INT4 checkExistBadBin = 0; checkExistBadBin < binCount; checkExistBadBin++)
+      {
+	if (badBin < 0)
+	  {
+	    LogPrintf ( LOG_CRITICAL, "badBin %d negative", badBin);
+	    XLAL_ERROR(XLAL_ETYPE);
+	  }
+	if (badBinData->data[checkExistBadBin] == (UINT4) badBin)
+	  checkIfBinAppeared++;
+      }
+    if (checkIfBinAppeared == 0)
+      {
+	badBinData->data[newBinCount] = badBin;
+	newBinCount++;
+      }
+  }
+
+  return newBinCount;
+
+}
