@@ -507,12 +507,14 @@ data!\n");
 
       fprintf(stderr, "Reading heterodyned data from %s.\n", inputParams.datafile);
 
-      /* read in header info */
+      /* read in header info (if not working on legacy files without the header) */
       CHAR headerdata[HEADERSIZE];
-      size_t rch = XLALFileRead((void*)&headerdata[0], sizeof(CHAR), HEADERSIZE, fpin);
-      if ( !rch ){
-        fprintf(stderr, "Error... problem reading in header data!\n");
-        exit(1);
+      if ( !inputParams.legacyinput ){
+        size_t rch = XLALFileRead((void*)&headerdata[0], sizeof(CHAR), HEADERSIZE, fpin);
+        if ( !rch ){
+          fprintf(stderr, "Error... problem reading in header data!\n");
+          exit(1);
+        }
       }
 
       /* read in file - depends on if file is binary or not */
@@ -815,11 +817,12 @@ void get_input_args(InputParams *inputParams, int argc, char *argv[]){
     { "binary-input",             no_argument,     NULL, 'B' },
     { "binary-output",            no_argument,     NULL, 'b' },
     { "gzip-output",              no_argument,     NULL, 'Z' },
+    { "legacy-input",             no_argument,     NULL, 'L' },
     { "verbose",                  no_argument,     NULL, 'v' },
     { 0, 0, 0, 0 }
   };
 
-  char args[] = "hi:p:z:f:g:k:s:r:d:c:o:e:S:t:l:R:C:F:O:T:m:G:H:M:ABbZv";
+  char args[] = "hi:p:z:f:g:k:s:r:d:c:o:e:S:t:l:R:C:F:O:T:m:G:H:M:ABbZLv";
   char *program = argv[0];
 
   /* set defaults */
@@ -829,8 +832,8 @@ void get_input_args(InputParams *inputParams, int argc, char *argv[]){
   inputParams->samplerate = 0.;
   inputParams->calibrate = 0; /* default is not to calibrate */
   inputParams->verbose = 0; /* default is not to do verbose */
-  inputParams->binaryinput = 0; /* default to NOT read in data from a binary
-file */
+  inputParams->binaryinput = 0; /* default to NOT read in data from a binary file */
+  inputParams->legacyinput = 0; /* default is that input files are not legacy files without the header information */
   inputParams->binaryoutput = 0; /* default is to output data as ASCII text */
   inputParams->gzipoutput = 0; /* default is to not gzip the output */
   inputParams->stddevthresh = 0.; /* default is not to threshold */
@@ -1031,6 +1034,9 @@ the pulsar parameter file */
       case 'M':
         inputParams->manualEpoch = atof(LALoptarg);
         break;
+      case 'L':
+        inputParams->legacyinput = 1;
+        break;
       case '?':
         fprintf(stderr, "unknown error while parsing options\n" );
       default:
@@ -1098,8 +1104,6 @@ void heterodyne_data(COMPLEX16TimeSeries *data, REAL8Vector *times,
   REAL8 filtphase=0.;
   UINT4 position=0, middle=0;
 
-  REAL8 lyr_pc = LAL_PC_SI/LAL_LYR_SI; /* light years per parsec */
-
   REAL8 om = PulsarGetREAL8ParamOrZero( hetParams.het, "WAVE_OM" ), omu = PulsarGetREAL8ParamOrZero( hetParams.hetUpdate, "WAVE_OM" );
 
   /* set the position, frequency and whitening epochs if not already set */
@@ -1142,22 +1146,16 @@ void heterodyne_data(COMPLEX16TimeSeries *data, REAL8Vector *times,
     baryinput.site.location[1] = hetParams.detector.location[1]/LAL_C_SI;
     baryinput.site.location[2] = hetParams.detector.location[2]/LAL_C_SI;
 
-    /* set 1/distance using parallax or distance value is given (try parallax first) - in 1/secs */
+    /* set 1/distance using parallax value if given - in 1/secs */
     if( hetParams.heterodyneflag != 2 && hetParams.heterodyneflag != 4 ){ /* not
       using updated params */
       REAL8 px = PulsarGetREAL8ParamOrZero( hetParams.het, "PX" );
-      REAL8 dist = PulsarGetREAL8ParamOrZero( hetParams.het, "DIST" );
-
-      if ( px != 0. ){ baryinput.dInv = ( 3600. / LAL_PI_180 )*px / (LAL_C_SI*lyr_pc); }
-      else if ( dist != 0. ){ baryinput.dInv = 1./(dist*1e3*LAL_C_SI*lyr_pc); }
+      if ( px != 0. ){ baryinput.dInv = px*(LAL_C_SI/LAL_AU_SI); }
       else { baryinput.dInv = 0.; } /* no parallax */
     }
     else{                                /* using updated params */
       REAL8 pxu = PulsarGetREAL8ParamOrZero( hetParams.hetUpdate, "PX" );
-      REAL8 distu = PulsarGetREAL8ParamOrZero( hetParams.hetUpdate, "DIST" );
-
-      if ( pxu != 0. ){ baryinput.dInv = ( 3600. / LAL_PI_180 )*pxu / (LAL_C_SI*lyr_pc); }
-      else if( distu != 0. ){ baryinput.dInv = 1./(distu*1e3*LAL_C_SI*lyr_pc); }
+      if ( pxu != 0. ){ baryinput.dInv = pxu*(LAL_C_SI/LAL_AU_SI); }
       else { baryinput.dInv = 0.; } /* no parallax */
     }
   }
