@@ -211,12 +211,24 @@ void initialise_algorithm( LALInferenceRunState *runState )
   if ( ppt != NULL ){
     FILE *timefile = NULL;
     UINT4 timenum = 1;
+
     ppt = LALInferenceGetProcParamVal( commandLine, "--outfile" );
+    if ( !ppt ){
+      XLAL_ERROR_VOID(XLAL_EFUNC, "Error... no output file is specified!");
+    }
 
-    if ( !ppt ){ XLAL_ERROR_VOID(XLAL_EFUNC, "Error... no output file is specified!"); }
-
-    CHAR outtimefile[256] = "";
-    sprintf(outtimefile, "%s_timings", ppt->value);
+    CHAR *outtimefile = NULL;
+    outtimefile = XLALStringDuplicate( ppt->value );
+    /* strip the file extension */
+    CHAR *dotloc = strrchr(outtimefile, '.');
+    CHAR *slashloc = strrchr(outtimefile, '/');
+    if ( dotloc != NULL ){
+      if ( slashloc != NULL ){ /* check dot is after any filename seperator */
+        if( slashloc < dotloc ){ *dotloc = '\0'; }
+      }
+      else{ *dotloc = '\0'; }
+    }
+    outtimefile = XLALStringAppend( outtimefile, "_timings" );
 
     if ( ( timefile = fopen(outtimefile, "w") ) == NULL ){
       fprintf(stderr, "Warning... cannot create a timing file, so proceeding without timings\n");
@@ -225,22 +237,11 @@ void initialise_algorithm( LALInferenceRunState *runState )
       LALInferenceAddVariable( runState->algorithmParams, "timefile", &timefile, LALINFERENCE_void_ptr_t, LALINFERENCE_PARAM_FIXED );
       LALInferenceAddVariable( runState->algorithmParams, "timenum", &timenum, LALINFERENCE_UINT4_t, LALINFERENCE_PARAM_FIXED );
     }
+    XLALFree( outtimefile );
   }
 
-  INT4 outputallparams=0;
-  if ( LALInferenceGetProcParamVal( runState->commandLine, "--output-all-params" ))
-  {
-    outputallparams=1;
-  }
-  LALInferenceAddVariable(runState->algorithmParams,"output_all_params",&outputallparams, LALINFERENCE_INT4_t,LALINFERENCE_PARAM_FIXED);
-
-  
   /* log samples */
-#ifdef HAVE_LIBLALXML
-  runState->logsample = LogSampleToArray;
-#else
-  runState->logsample = LogSampleToFile;
-#endif
+  runState->logsample = LALInferenceLogSampleToArray;
 
   return;
 }
@@ -371,12 +372,14 @@ void add_initial_variables( LALInferenceVariables *ini, PulsarParameters *pars )
   if ( PulsarCheckParam(pars, "F") ){ /* frequency and frequency derivative parameters */
     UINT4 i = 0;
     REAL8Vector *freqs = PulsarGetREAL8VectorParam( pars, "F" );
-    /* add each frequency and derivative value as a seperate parameter */
+    /* add each frequency and derivative value as a seperate parameter (also set a value that is the FIXED value to be used for calculating phase differences) */
     for ( i = 0; i < freqs->length; i++ ){
       CHAR varname[256];
       snprintf(varname, sizeof(varname), "F%u", i);
       REAL8 fval = PulsarGetREAL8VectorParamIndividual( pars, varname );
       LALInferenceAddVariable( ini, varname, &fval, LALINFERENCE_REAL8_t, LALINFERENCE_PARAM_FIXED );
+      snprintf(varname, sizeof(varname), "F%u_FIXED", i);
+      LALInferenceAddVariable( ini, varname, &fval, LALINFERENCE_REAL8_t, LALINFERENCE_PARAM_FIXED ); /* add FIXED value */
     }
     /* add value with the number of FB parameters given */
     LALInferenceAddVariable( ini, "FREQNUM", &i, LALINFERENCE_UINT4_t, LALINFERENCE_PARAM_FIXED );
@@ -418,8 +421,9 @@ void add_initial_variables( LALInferenceVariables *ini, PulsarParameters *pars )
   add_variable_parameter( pars, ini, "PMDEC", LALINFERENCE_PARAM_FIXED );
   add_variable_parameter( pars, ini, "POSEPOCH", LALINFERENCE_PARAM_FIXED );
 
-  /* source distance */
+  /* source distance and parallax */
   add_variable_parameter( pars, ini, "DIST", LALINFERENCE_PARAM_FIXED );
+  add_variable_parameter( pars, ini, "PX", LALINFERENCE_PARAM_FIXED );
 
   /* only add binary system parameters if required */
   if ( PulsarCheckParam( pars, "BINARY" ) ){
@@ -504,14 +508,26 @@ void add_initial_variables( LALInferenceVariables *ini, PulsarParameters *pars )
  * \brief Sets up the parameters to be searched over and their prior ranges
  *
  * This function sets up any parameters that you require the code to search over and specifies the prior range and type
- * for each. This information is contained in a prior file specified by the command line argument \c prior-file. This
- * file should contain four columns: the first has the name of a parameter to be searched over; the second has the prior
- * type (e.g. "uniform" for a prior that is flat over the given range, or "gaussian" with a certain mean and standard
- * deviation, or "fermidirac" for a Fermi-Dirac distribution defined by a sigma and r parameters (where r is mu/sigma),
- * or "predefined", which means that the prior for that variable is already hardcoded into the prior function);
- * the third has the lower limit, or mean, of the prior, for "uniform"/"predefined" and "gaussian" priors respectively;
- * and the fourth has the upper limit, or standard deviation, for "uniform"/"predefined" and "gaussian" priors respectively.
- * E.g.
+ * for each. This information is contained in a prior file specified by the command line argument \c prior-file. There
+ * are currently five different allowed prior distributions:
+ *   - "uniform": A flat distribution between a minimum and maximum range (with zero probabiility outside that range);
+ *   - "gaussian": A Gaussian/Normal distribution defined by a mean and standard devaition;
+ *   - "fermidirac": A Fermi-Dirac-like distribution defined by a knee and attenuation width;
+ *   - "gmm": A one-dimensional Gaussian Mixture model defined by the means, standard deviations and weights of each mode.
+ *   - "loguniform": A flat distribution in log-space (proportional to the inverse of the value in non-log-space)
+ * For all priors the first two columns of the prior definition should be:
+ *   -# the name of a parameter to be searched over, and
+ *   -# the prior type (e.g. "uniform", "gaussian", "fermidirac", "gmm" or "loguniform".
+ * For the "uniform, "loguniform", "gaussian", "fermidirac" priors the prior file should contain the a further two columns containing:
+ *   -# the lower limit ("uniform" and "loguniform"), mean ("gaussian"), or sigma ("fermidirac") of the distribution;
+ *   -# the upper limit ("uniform" and "loguniform"), standard deviation ("gaussian"), or r ("fermidirac") of the distribution.
+ * For the "gmm" prior the file should contain at least a further four columns. The third column gives the number of modes
+ * for the mixture. For each mode there should then be a tuple of numbers giving the mean, standard deviation and weight
+ * of each mode (weights can be relative weights as they will be normalised when parsed by the code). Finally, there can
+ * be two additional columns giving lower and upper limits of the distribution (note that the weights given are weights
+ * assuming that there are no bounds on the distributions).
+ *
+ * Some examples of files are:
  * \code
  * H0 uniform 0 1e-21
  * PHI0 uniform 0 3.14159265359
@@ -520,9 +536,16 @@ void add_initial_variables( LALInferenceVariables *ini, PulsarParameters *pars )
  * \endcode
  * or
  * \code
- * H0 uniform 0 1e-21
+ * H0 fermidirac 3.3e-26 9.16
  * PHI0 uniform 0 3.14159265359
- * IOTA predefined 0 3.14159265359
+ * IOTA uniform 0 3.14159265359
+ * PSI uniform -0.785398163397448 0.785398163397448
+ * \endcode
+ * or
+ * \code
+ * H0 gmm 3 1e-25 2e-24 1.5 2e-25 3e-24 2.5 1e-26 1e-25 0.2 0.0 1e-20
+ * PHI0 uniform 0 3.14159265359
+ * COSIOTA uniform -1 1
  * PSI uniform -0.785398163397448 0.785398163397448
  * \endcode
  *
@@ -542,13 +565,16 @@ void initialise_prior( LALInferenceRunState *runState )
   LALInferenceThreadState *threadState = runState->threads[0];
   ProcessParamsTable *ppt;
   ProcessParamsTable *commandLine = runState->commandLine;
-  FILE *fp=NULL;
 
-  CHAR tempPar[VARNAME_MAX] = "", tempPrior[VARNAME_MAX] = "";
+  CHAR *tempPar = NULL, *tempPrior = NULL;
   REAL8 low, high;
 
   LALInferenceIFOModel *ifo = threadState->model->ifo;
 
+  CHAR *filebuf =  NULL; /* buffer to store prior file */
+  TokenList *tlist = NULL;
+  UINT4 k = 0, nvals = 0;
+
   /* parameters in correlation matrix */
   LALStringVector *corParams = NULL;
   REAL8Array *corMat = NULL;
@@ -567,45 +593,113 @@ void initialise_prior( LALInferenceRunState *runState )
     exit(0);
   }
 
-  /* open file */
-  if( (fp = fopen(propfile, "r")) == NULL ){
-    fprintf(stderr, "Error... Could not open prior file %s.\n", propfile);
-    exit(3);
+  /* read in prior file and separate lines */
+  filebuf = XLALFileLoad( propfile );
+  if ( XLALCreateTokenList( &tlist, filebuf, "\n" ) != XLAL_SUCCESS ){
+    XLAL_ERROR_VOID(XLAL_EINVAL, "Error... could not convert data into separate lines.\n");
   }
 
-  while(fscanf(fp, "%s %s %lf %lf", tempPar, tempPrior, &low, &high) != EOF){
-    INT4 isthere = 0, i = 0;
-
+  /* parse through priors */
+  for ( k = 0; k < tlist->nTokens; k++ ){
+    UINT4 isthere = 0, i = 0;
     LALInferenceParamVaryType varyType;
 
-    /* convert tempPar to all uppercase letters */
-    strtoupper( tempPar );
+    /* check for comment line starting with '#' or '%' */
+    if ( !strcmp(tlist->tokens[k], "#") || !strcmp(tlist->tokens[k], "%") ){ continue; }
 
-    if ( !strcmp(tempPrior, "uniform") || !strcmp(tempPrior, "predefined") ){
-      if( high < low ){
-        XLAL_ERROR_VOID( XLAL_EINVAL, "Error... In %s the %s parameters ranges are wrongly set.", propfile, tempPar );
-      }
+    /* check the number of values in the line by counting the whitespace separated values */
+    nvals = 0;
+    TokenList *tline = NULL;
+    XLALCreateTokenList( &tline, tlist->tokens[k], " \t" );
+    nvals = tline->nTokens;
+
+    if ( nvals < 2 ){
+      fprintf(stderr, "Warning... number of values ('%d') on line '%d' in prior file is different than expected:\n\t'%s'", nvals, k+1, tlist->tokens[k]);
+      XLALDestroyTokenList( tline );
+      continue;
     }
 
-    if ( strcmp(tempPrior, "uniform") && strcmp(tempPrior, "predefined") && strcmp(tempPrior, "gaussian") && strcmp(tempPrior, "fermidirac") ){
-      XLAL_ERROR_VOID( XLAL_EINVAL, "Error... prior type '%s' not recognised", tempPrior );
+    tempPar = XLALStringDuplicate( tline->tokens[0] );
+    strtoupper( tempPar ); /* convert tempPar to all uppercase letters */
+    tempPrior = XLALStringDuplicate( tline->tokens[1] );
+
+    /* gaussian, uniform, loguniform and fermi-dirac priors should all have four values to a line */
+    if ( nvals == 4 ){
+      low = atof(tline->tokens[2]);
+      high = atof(tline->tokens[3]);
+
+      if ( !strcmp(tempPrior, "uniform") || !strcmp(tempPrior, "loguniform") ){
+        if( high < low ){
+          XLAL_ERROR_VOID( XLAL_EINVAL, "Error... In %s the %s parameters ranges are wrongly set.", propfile, tempPar );
+        }
+      }
+
+      if ( strcmp(tempPrior, "uniform") && strcmp(tempPrior, "loguniform") && strcmp(tempPrior, "gaussian") && strcmp(tempPrior, "fermidirac") ){
+        XLAL_ERROR_VOID( XLAL_EINVAL, "Error... prior type '%s' not recognised", tempPrior );
+      }
+
+      /* Add the prior variables */
+      if ( !strcmp(tempPrior, "uniform") ){
+        LALInferenceAddMinMaxPrior( runState->priorArgs, tempPar, &low, &high, LALINFERENCE_REAL8_t );
+      }
+      else if ( !strcmp(tempPrior, "loguniform") ){
+        LALInferenceAddLogUniformPrior( runState->priorArgs, tempPar, &low, &high, LALINFERENCE_REAL8_t );
+      }
+      else if( !strcmp(tempPrior, "gaussian") ){
+        LALInferenceAddGaussianPrior( runState->priorArgs, tempPar, &low, &high, LALINFERENCE_REAL8_t );
+      }
+      else if( !strcmp(tempPrior, "fermidirac") ){
+        LALInferenceAddFermiDiracPrior( runState->priorArgs, tempPar, &low, &high, LALINFERENCE_REAL8_t );
+      }
+    }
+    else if ( nvals > 4 ){
+      /* check if using a 1D Gaussian Mixture Model prior e.g.:
+       * iota gmm 2 0.5 0.01 1.0 1.5 0.01 1.0 0.0 3.14
+       * where the third value is the number of modes, followed by tuples of means, standard deviations and weights for each mode
+       * and finally (if required two values giving minimum and maximum limits for the prior).
+       */
+      if ( !strcmp(tempPrior, "gmm") ){
+        UINT4 nmodes = (UINT4)atoi( tline->tokens[2] );
+
+        if ( nvals - 2 < 3*nmodes ){
+          XLAL_ERROR_VOID( XLAL_EINVAL, "Error... GMM prior is not properly defined" );
+        }
+
+        /* get Gaussian mode means, standard deviations and weights */
+        REAL8Vector *gmmsigmas = NULL, *gmmmus = NULL, *gmmweights = NULL;
+        gmmsigmas = XLALCreateREAL8Vector( nmodes );
+        gmmmus = XLALCreateREAL8Vector( nmodes );
+        gmmweights = XLALCreateREAL8Vector( nmodes );
+        for ( i = 0; i < nmodes; i++ ){
+          gmmmus->data[i] = atof( tline->tokens[3+3*i] );
+          gmmsigmas->data[i] = atof( tline->tokens[3+3*i+1] );
+          gmmweights->data[i] = atof( tline->tokens[3+3*i+2] );
+        }
+
+        REAL8 minval = -INFINITY, maxval = INFINITY;
+        /* check if minimum and maximum bounds are specified */
+        if ( nvals > 3+3*nmodes ) { minval = atof(tline->tokens[3+3*nmodes]); }
+        if ( nvals > 3+3*nmodes+1 ) { maxval = atof(tline->tokens[3+3*nmodes+1]); }
+        if ( nvals > 3+3*nmodes+2 ){
+          fprintf(stderr, "Warning... additional unnecessary values given in GMM prior\n");
+        }
+
+        LALInferenceAdd1DGMMPrior( runState->priorArgs, tempPar, &gmmmus, &gmmsigmas, &gmmweights, &minval, &maxval );
+      }
+      else{
+         XLAL_ERROR_VOID( XLAL_EINVAL, "Error... prior type '%s' not recognised", tempPrior );
+      }
+    }
+    else{
+      fprintf(stderr, "Warning... number of values ('%d') on line '%d' in prior file is different than expected:\n\t'%s'\n", nvals, k+1, tlist->tokens[k]);
+      XLALDestroyTokenList( tline );
+      continue;
     }
 
     /* set variable type to LINEAR (as they are initialised as FIXED) */
     varyType = LALINFERENCE_PARAM_LINEAR;
     LALInferenceSetParamVaryType( threadState->currentParams, tempPar, varyType );
 
-    /* Add the prior variables */
-    if ( !strcmp(tempPrior, "uniform") || !strcmp(tempPrior, "predefined") ){
-      LALInferenceAddMinMaxPrior( runState->priorArgs, tempPar, &low, &high, LALINFERENCE_REAL8_t );
-    }
-    else if( !strcmp(tempPrior, "gaussian") ){
-      LALInferenceAddGaussianPrior( runState->priorArgs, tempPar, &low, &high, LALINFERENCE_REAL8_t );
-    }
-    else if( !strcmp(tempPrior, "fermidirac") ){
-      LALInferenceAddFermiDiracPrior( runState->priorArgs, tempPar, &low, &high, LALINFERENCE_REAL8_t );
-    }
-
     /* if there is a phase parameter defined in the proposal then set varyphase to 1 */
     for ( i = 0; i < NUMAMPPARS; i++ ){
       if ( !strcmp(tempPar, amppars[i]) ){
@@ -630,8 +724,13 @@ void initialise_prior( LALInferenceRunState *runState )
         break;
       }
     }
+
+    XLALDestroyTokenList( tline );
   }
 
+  XLALDestroyTokenList( tlist );
+  XLALFree( filebuf );
+
   LALInferenceIFOModel *ifotemp = ifo;
   while( ifotemp ){
     /* add in variables to say whether phase, sky position and binary parameter are varying */
@@ -1261,8 +1360,7 @@ void sum_data( LALInferenceRunState *runState ){
         check_and_add_fixed_variable( ifomodel->params, "sumBLWhite", &sumBLWhite, LALINFERENCE_REAL8Vector_t );
       }
     }
-    else{ /* add parameter defining the usage of RQO here (as this is after any injection generation, which
-           * would fail if this was set */
+    else{ /* add parameter defining the usage of RQO here (as this is after any injection generation, which would fail if this was set */
       LALInferenceAddVariable( ifomodel->params, "roq", &roq, LALINFERENCE_UINT4_t, LALINFERENCE_PARAM_FIXED );
     }
 
@@ -1276,142 +1374,6 @@ void sum_data( LALInferenceRunState *runState ){
 }
 
 
-/**
- * \brief Print any non-fixed sample to file (based on \c LALInferencePrintSampleNonFixed)
- */
-static void PrintNonFixedSample(FILE *fp, LALInferenceVariables *sample){
-  UINT4 i;
-  UINT4Vector *v=NULL;
-
-  if(sample==NULL) { return; }
-
-  LALInferenceVariableItem *ptr=sample->head;
-  if(fp==NULL) { return; }
-
-  while(ptr!=NULL) {
-    if (LALInferenceGetVariableVaryType(sample, ptr->name) != LALINFERENCE_PARAM_FIXED && ptr->type != LALINFERENCE_gslMatrix_t ) {
-      switch (ptr->type) {
-        case LALINFERENCE_INT4_t:
-          fprintf(fp, "%"LAL_INT4_FORMAT, *(INT4 *) ptr->value);
-          break;
-        case LALINFERENCE_INT8_t:
-          fprintf(fp, "%"LAL_INT8_FORMAT, *(INT8 *) ptr->value);
-          break;
-        case LALINFERENCE_UINT4_t:
-          fprintf(fp, "%"LAL_UINT4_FORMAT, *(UINT4 *) ptr->value);
-          break;
-        case LALINFERENCE_REAL4_t:
-          fprintf(fp, "%9.20e", *(REAL4 *) ptr->value);
-          break;
-        case LALINFERENCE_REAL8_t:
-          fprintf(fp, "%9.20le", *(REAL8 *) ptr->value);
-          break;
-        case LALINFERENCE_COMPLEX8_t:
-          fprintf(fp, "%e + i*%e", (REAL4) crealf(*(COMPLEX8 *) ptr->value), (REAL4) cimagf(*(COMPLEX8 *) ptr->value));
-          break;
-        case LALINFERENCE_COMPLEX16_t:
-          fprintf(fp, "%e + i*%e", (REAL8) creal(*(COMPLEX16 *) ptr->value), (REAL8) cimag(*(COMPLEX16 *) ptr->value));
-          break;
-        case LALINFERENCE_UINT4Vector_t:
-          v = *((UINT4Vector **)ptr->value);
-          for(i=0;i<v->length;i++){
-            fprintf(fp,"%11.7f",(REAL8)v->data[i]);
-            if( i!=(UINT4)(v->length-1) ) { fprintf(fp,"\t"); }
-          }
-          break;
-        default:
-          fprintf(stdout, "<can't print>");
-      }
-      fprintf(fp,"\t");
-    }
-    ptr=ptr->next;
-  }
-  return;
-}
-
-
-/**
- * \brief Print out only the variable (i.e. non-fixed) parameters to the file
- *
- * If the command line argument \c --output-all-params is given then this function
- * will output all parameters that have been stored, but by default it will only
- * output variable parameters to the nested samples to a file.
- */
-void LogSampleToFile(LALInferenceVariables *algorithmParams, LALInferenceVariables *vars)
-{
-
-  FILE *outfile = NULL;
-  if( LALInferenceCheckVariable(algorithmParams,"outfile") ){
-    outfile = *(FILE **)LALInferenceGetVariable(algorithmParams,"outfile");
-  }
-
-  /* Write out old sample */
-  if( outfile == NULL ) { return; }
-  LALInferenceSortVariablesByName(vars);
-
-  /* only write out non-fixed samples if required */
-  if(LALInferenceCheckVariable(algorithmParams,"output_all_params")&&LALInferenceGetINT4Variable(algorithmParams,"output_all_params")){
-    LALInferencePrintSample(outfile, vars);
-  }
-  else{
-    /* default to writing out only the non-fixed (i.e. variable) parameters */
-    PrintNonFixedSample(outfile, vars);
-  }
-  fprintf(outfile,"\n");
-
-  return;
-}
-
-
-/**
- * \brief Print out only the variable (i.e. non-fixed) parameters to the file whilst also creating
- * an array for all parameters
- *
- * This function (which is used if the XML library is present) will only output variable
- * parameters.
- */
-void LogSampleToArray(LALInferenceVariables *algorithmParams, LALInferenceVariables *vars)
-{
-  LALInferenceVariables **output_array = NULL;
-  UINT4 N_output_array = 0;
-  LALInferenceSortVariablesByName(vars);
-
-  LogSampleToFile(algorithmParams, vars);
-
-  /* Set up the array if it is not already allocated */
-  if(LALInferenceCheckVariable(algorithmParams,"outputarray")){
-    output_array = *(LALInferenceVariables ***)LALInferenceGetVariable(algorithmParams,"outputarray");
-  }
-  else{
-    LALInferenceAddVariable(algorithmParams,"outputarray",&output_array,LALINFERENCE_void_ptr_t,LALINFERENCE_PARAM_OUTPUT);
-  }
-
-  if(LALInferenceCheckVariable(algorithmParams,"N_outputarray")){
-    N_output_array = *(INT4 *)LALInferenceGetVariable(algorithmParams,"N_outputarray");
-  }
-  else{
-    LALInferenceAddVariable(algorithmParams,"N_outputarray",&N_output_array,LALINFERENCE_INT4_t,LALINFERENCE_PARAM_OUTPUT);
-  }
-
-  /* Expand the array for new sample */
-  output_array = XLALRealloc(output_array, (N_output_array+1) *sizeof(LALInferenceVariables *));
-  if( !output_array ){
-    XLAL_ERROR_VOID(XLAL_EFAULT, "Unable to allocate array for samples.");
-  }
-  else{
-    /* Save sample and update */
-    output_array[N_output_array] = XLALCalloc(1,sizeof(LALInferenceVariables));
-    LALInferenceCopyVariables(vars, output_array[N_output_array]);
-    N_output_array++;
-
-    LALInferenceSetVariable(algorithmParams,"outputarray",&output_array);
-    LALInferenceSetVariable(algorithmParams,"N_outputarray",&N_output_array);
-  }
-
-  return;
-}
-
-
 void initialise_threads(LALInferenceRunState *state, INT4 nthreads){
   INT4 i,randomseed;
   LALInferenceThreadState *thread;
