@@ -62,14 +62,24 @@ void inject_signal( LALInferenceRunState *runState ){
   REAL8 snrscale = 0;
 
   ppt = LALInferenceGetProcParamVal( commandLine, "--outfile" );
-  if( !ppt ){ XLAL_ERROR_VOID( XLAL_EINVAL, "Error... no output file specified!" ); }
+  if ( !ppt ){ XLAL_ERROR_VOID( XLAL_EINVAL, "Error... no output file specified!" ); }
 
   snrfile = XLALStringDuplicate( ppt->value );
+  /* strip the file extension */
+  CHAR *dotloc = strrchr(snrfile, '.');
+  CHAR *slashloc = strrchr(snrfile, '/');
+  if ( dotloc != NULL ){
+    if ( slashloc != NULL ){ /* check dot is after any filename seperator */
+      if( slashloc < dotloc ){ *dotloc = '\0'; }
+    }
+    else{ *dotloc = '\0'; }
+  }
   snrfile = XLALStringAppend( snrfile, "_SNR" );
 
   if( (fpsnr = fopen(snrfile, "w")) == NULL ){
     XLAL_ERROR_VOID( XLAL_EIO, "Error... cannot open output SNR file!");
   }
+  XLALFree( snrfile );
 
   ppt = LALInferenceGetProcParamVal( commandLine, "--inject-file" );
   if( ppt ){
@@ -82,6 +92,7 @@ void inject_signal( LALInferenceRunState *runState ){
 
     /* read in injection parameter file */
     injpars = XLALReadTEMPOParFileNew( injectfile );
+    XLALFree( injectfile );
 
     /* check RA and DEC are set (if only RAJ and DECJ are given in the par file) */
     if ( !PulsarCheckParam( injpars, "RA" ) ){
@@ -120,11 +131,25 @@ void inject_signal( LALInferenceRunState *runState ){
 
   /* create signal to inject */
   /* for injection always attempt to include the signal phase model even if the search is not going to be over phase */
-  INT4 varyphase = 1;
+  INT4 varyphase = 1, varyskypos = 1, varybinary = 1;
+  while ( ifo_model ) {
+    if ( !LALInferenceCheckVariable( ifo_model->params, "varyphase" ) ){
+      LALInferenceAddVariable( ifo_model->params, "varyphase", &varyphase, LALINFERENCE_INT4_t, LALINFERENCE_PARAM_FIXED );
+      varyphase = 0; /* set to zero so varyphase is removed after the model is created */
 
-  if ( !LALInferenceCheckVariable( ifo_model->params, "varyphase" ) ){
-    LALInferenceAddVariable( ifo_model->params, "varyphase", &varyphase, LALINFERENCE_INT4_t, LALINFERENCE_PARAM_FIXED );
-    varyphase = 0; /* set to zero so varyphase is removed after the model is created */
+      if ( !LALInferenceCheckVariable( ifo_model->params, "varyskypos" ) ){
+        LALInferenceAddVariable( ifo_model->params, "varyskypos", &varyskypos, LALINFERENCE_INT4_t, LALINFERENCE_PARAM_FIXED );
+        varyskypos = 0;
+      }
+
+      if ( PulsarCheckParam( injpars, "BINARY" ) ){
+        if ( !LALInferenceCheckVariable( ifo_model->params, "varybinary" ) ){
+          LALInferenceAddVariable( ifo_model->params, "varybinary", &varybinary, LALINFERENCE_INT4_t, LALINFERENCE_PARAM_FIXED );
+          varybinary = 0;
+        }
+      }
+    }
+    ifo_model = runState->threads[0]->model->ifo;
   }
 
   /* check whether to inject a non-GR signal (the default will ALWAYS be GR) */
@@ -180,15 +205,7 @@ void inject_signal( LALInferenceRunState *runState ){
   while ( data ){
     REAL8 snrval = 0.;
 
-    if ( !LALInferenceCheckVariable( ifo_model->params, "varyphase" ) ){
-      LALInferenceAddVariable( ifo_model->params, "varyphase", &varyphase, LALINFERENCE_INT4_t, LALINFERENCE_PARAM_FIXED );
-    }
-
     snrval = calculate_time_domain_snr( data, ifo_model );
-
-    /* reset varyphase to its original value */
-    if ( !varyphase ){ LALInferenceRemoveVariable( ifo_model->params, "varyphase" ); }
-
     snrmulti += SQUARE(snrval);
 
     /* if not scaling print out individual detector/datastream SNRs */
@@ -218,18 +235,10 @@ void inject_signal( LALInferenceRunState *runState ){
     PulsarAddParam( injpars, "C22", &C22, PULSARTYPE_REAL8_t );
     PulsarAddParam( injpars, "C21", &C21, PULSARTYPE_REAL8_t );
 
-    /* recreate the signal with scaled amplitude */
-    varyphase = 1;
-
     /* reset to head */
     ifo_model = runState->threads[0]->model->ifo;
     data = runState->data;
 
-    if ( !LALInferenceCheckVariable( ifo_model->params, "varyphase" ) ){
-      LALInferenceAddVariable( ifo_model->params, "varyphase", &varyphase, LALINFERENCE_INT4_t, LALINFERENCE_PARAM_FIXED );
-      varyphase = 0; /* set to zero so varyphase is removed after the model is created */
-    }
-
     pulsar_model( injpars, ifo_model );
 
     /* get new summed data for use in SNR calculation */
@@ -244,16 +253,8 @@ void inject_signal( LALInferenceRunState *runState ){
     while( data ){
       REAL8 snrval = 0.;
 
-      if ( !LALInferenceCheckVariable( ifo_model->params, "varyphase" ) ){
-        LALInferenceAddVariable( ifo_model->params, "varyphase", &varyphase, LALINFERENCE_INT4_t, LALINFERENCE_PARAM_FIXED );
-      }
-
       /* recalculate the SNR */
       snrval = calculate_time_domain_snr( data, ifo_model );
-
-      /* reset varyphase to its original value */
-      if ( !varyphase ){ LALInferenceRemoveVariable( ifo_model->params, "varyphase" ); }
-
       snrmulti += SQUARE(snrval);
 
       fprintf(fpsnr, "%s\t%.3lf\t%le\n", data->name, freqFactors->data[ndats%(INT4)freqFactors->length], snrval);
@@ -311,6 +312,7 @@ void inject_signal( LALInferenceRunState *runState ){
       if ( (fpso = fopen(signalonly, "w")) == NULL ){
         fprintf(stderr, "Non-fatal error... unable to open file %s to output injection\n", signalonly);
       }
+      XLALFree( outfile );
     }
 
     /* add the signal to the data */
@@ -350,6 +352,9 @@ void inject_signal( LALInferenceRunState *runState ){
       while ( ifo_model ){
         LALInferenceAddVariable( ifo_model->params, "nonGR", &nonGRval, LALINFERENCE_INT4_t, LALINFERENCE_PARAM_FIXED );
         ifo_model = ifo_model->next;
+        if ( !varyphase ){ LALInferenceRemoveVariable( ifo_model->params, "varyphase" ); }
+        if ( !varyskypos ){ LALInferenceRemoveVariable( ifo_model->params, "varyskypos" ); }
+        if ( !varybinary ){ LALInferenceRemoveVariable( ifo_model->params, "varybinary" ); }
       }
       ifo_model = runState->threads[0]->model->ifo;
     }
@@ -357,6 +362,9 @@ void inject_signal( LALInferenceRunState *runState ){
   else {
     while ( ifo_model ){
       LALInferenceRemoveVariable(ifo_model->params, "nonGR");
+      if ( !varyphase ){ LALInferenceRemoveVariable( ifo_model->params, "varyphase" ); }
+      if ( !varyskypos ){ LALInferenceRemoveVariable( ifo_model->params, "varyskypos" ); }
+      if ( !varybinary ){ LALInferenceRemoveVariable( ifo_model->params, "varybinary" ); }
       ifo_model = ifo_model->next;
     }
     ifo_model = runState->threads[0]->model->ifo;
@@ -529,7 +537,7 @@ REAL8 calculate_time_domain_snr( LALInferenceIFOData *data, LALInferenceIFOModel
  */
 void get_loudest_snr( LALInferenceRunState *runState ){
   INT4 ndats = 0;
-  UINT4 roq = 0;
+  UINT4 roq = 0; // i = 0;
   INT4 Nlive = *(INT4 *)LALInferenceGetVariable( runState->algorithmParams, "Nlive" );
   REAL8 snrmulti = 0.;
   REAL8Vector *freqFactors = NULL;
@@ -554,8 +562,6 @@ void get_loudest_snr( LALInferenceRunState *runState ){
     roq = 1;
 
     while ( ifo_model ){
-      UINT4 varyphase = 1; /* must set varyphase */
-
       REAL8Vector *sidtime = *(REAL8Vector **)LALInferenceGetVariable( ifo_model->params, "siderealDayFull" );
       LALInferenceRemoveVariable( ifo_model->params, "siderealDay" );
       LALInferenceAddVariable( ifo_model->params, "siderealDay", &sidtime, LALINFERENCE_REAL8Vector_t, LALINFERENCE_PARAM_FIXED );
@@ -563,20 +569,13 @@ void get_loudest_snr( LALInferenceRunState *runState ){
       LIGOTimeGPSVector *timestamps = *(LIGOTimeGPSVector **)LALInferenceGetVariable( ifo_model->params, "timeStampVectorFull" );
       XLALDestroyTimestampVector( ifo_model->times );
       ifo_model->times = timestamps;
-
-      REAL8TimeSeries *timedata = *(REAL8TimeSeries **)LALInferenceGetVariable( ifo_model->params, "timeDataFull" );
-      XLALDestroyREAL8TimeSeries( ifo_model->timeData );
-      ifo_model->timeData = timedata;
+      //fprintf(stderr, "timestamps->length = %d, ifo_model->times->data[0] = %d, ifo_model->times->data[-1] = %d\n", timestamps->length, ifo_model->times->data[0].gpsSeconds, ifo_model->times->data[timestamps->length-1].gpsSeconds);
 
       ifo_model->compTimeSignal = XLALResizeCOMPLEX16TimeSeries( ifo_model->compTimeSignal, 0, ifo_model->times->length );
 
       /* remove the ROQ variable for calculating the likelihood */
       LALInferenceRemoveVariable( ifo_model->params, "roq" );
 
-      if ( !LALInferenceCheckVariable( ifo_model->params, "varyphase" ) ){
-        LALInferenceAddVariable( ifo_model->params, "varyphase", &varyphase, LALINFERENCE_UINT4_t, LALINFERENCE_PARAM_FIXED );
-      }
-
       if ( LALInferenceCheckVariable( ifo_model->params, "ssb_delays" ) ){
         REAL8Vector *ssbdelays = *(REAL8Vector **)LALInferenceGetVariable( ifo_model->params, "ssb_delays_full" );
         LALInferenceRemoveVariable( ifo_model->params, "ssb_delays" );
@@ -589,6 +588,12 @@ void get_loudest_snr( LALInferenceRunState *runState ){
         LALInferenceAddVariable( ifo_model->params, "bsb_delays", &bsbdelays, LALINFERENCE_REAL8Vector_t, LALINFERENCE_PARAM_FIXED );
       }
 
+      /* make sure varyphase is set (this is required if having used ROQ for a non-varyphase model) */
+      if ( !LALInferenceCheckVariable( ifo_model->params, "varyphase" ) ){
+        UINT4 varyphasetmp = 1;
+        LALInferenceAddVariable( ifo_model->params, "varyphase", &varyphasetmp, LALINFERENCE_UINT4_t, LALINFERENCE_PARAM_FIXED );
+      }
+
       ifo_model = ifo_model->next;
     }
   }
@@ -616,12 +621,18 @@ void get_loudest_snr( LALInferenceRunState *runState ){
 
   /* setup output file */
   ppt = LALInferenceGetProcParamVal( commandLine, "--outfile" );
-  if( !ppt ){
-    fprintf(stderr, "Error... no output file specified!\n");
-    exit(0);
-  }
+  if ( !ppt ){ XLAL_ERROR_VOID(XLAL_EIO, "Error... no output file specified!\n"); }
 
   snrfile = XLALStringDuplicate( ppt->value );
+  /* strip the file extension */
+  CHAR *dotloc = strrchr(snrfile, '.');
+  CHAR *slashloc = strrchr(snrfile, '/');
+  if ( dotloc != NULL ){
+    if ( slashloc != NULL ){ /* check dot is after any filename seperator */
+      if( slashloc < dotloc ){ *dotloc = '\0'; }
+    }
+    else{ *dotloc = '\0'; }
+  }
   snrfile = XLALStringAppend( snrfile, "_SNR" );
 
   /* append to previous injection SNR file if it exists */
@@ -629,6 +640,7 @@ void get_loudest_snr( LALInferenceRunState *runState ){
     fprintf(stderr, "Error... cannot open output SNR file!\n");
     exit(0);
   }
+  XLALFree( snrfile );
 
   /* get SNR of loudest point and print out to file */
   data = runState->data;
