@@ -33,6 +33,7 @@ import os
 import numpy as np
 import struct
 import re
+import h5py
 
 from scipy.integrate import cumtrapz
 from scipy.interpolate import interp1d
@@ -231,9 +232,9 @@ def pferrs(porf, porferr, pdorfd=None, pdorfderr=None):
 
 # class to read in a pulsar par file - this is heavily based on the function
 # in parfile.py in PRESTO
-float_keys = ["F", "F0", "F1", "F2", "F3", "F4", "F5", "F6", "F7", "F8", "F9",
+float_keys = ["F", "F0", "F1", "F2", "F3", "F4", "F5", "F6", "F7", "F8", "F9", "F10",
               "PEPOCH", "POSEPOCH", "DM", "START", "FINISH", "NTOA",
-              "TRES", "TZRMJD", "TZRFRQ", "TZRSITE", "NITS", "DM",
+              "TRES", "TZRMJD", "TZRFRQ", "TZRSITE", "NITS",
               "A1", "XDOT", "E", "ECC", "EDOT", "T0", "PB", "PBDOT", "OM",
               "OMDOT", "EPS1", "EPS2", "EPS1DOT", "EPS2DOT", "TASC", "LAMBDA",
               "BETA", "RA_RAD", "DEC_RAD", "GAMMA", "SINI", "M2", "MTOT",
@@ -423,11 +424,11 @@ class psr_par:
 
     # masses
     for mass in ['M2', 'MTOT']:
-      if hasattr(self, 'M2'): # convert solar masses to kg
-        setattr(self, 'M2', self['M2']*MSUN)
+      if hasattr(self, mass): # convert solar masses to kg
+        setattr(self, mass, self[mass]*MSUN)
 
-        if hasattr(self, 'M2_ERR'):
-          setattr(self, 'M2_ERR', self['M2_ERR'] * MSUN)
+        if hasattr(self, mass+'_ERR'):
+          setattr(self, mass+'_ERR', self[mass+'_ERR'] * MSUN)
 
     # D_AOP
     if hasattr(self, 'D_AOP'): # convert from inverse arcsec to inverse radians
@@ -1298,7 +1299,7 @@ def plot_Bks_ASDs( Bkdata, delt=86400, plotpsds=True, plotfscan=False, removeout
       'font.size': 15 }
 
   matplotlib.rcParams.update(mplparams)
-  matplotlib.rcParams['text.latex.preamble']=r'\usepackage{nicefrac}'
+  matplotlib.rcParams['text.latex.preamble']=r'\usepackage{xfrac}'
 
   # ifos line colour specs
   coldict = {'H1': 'r', 'H2': 'c', 'L1': 'g', 'V1': 'b', 'G1': 'm'}
@@ -1416,9 +1417,9 @@ def plot_Bks_ASDs( Bkdata, delt=86400, plotpsds=True, plotfscan=False, removeout
             xl.append('0')
           else:
             if item < 0:
-              xl.append(r'$-\nicefrac{1}{%d}$' % (-1./item))
+              xl.append(r'$-\sfrac{1}{%d}$' % (-1./item))
             else:
-              xl.append(r'$\nicefrac{1}{%d}$' % (1./item))
+              xl.append(r'$\sfrac{1}{%d}$' % (1./item))
         ax.set_xticklabels(xl)
         #plt.setp(ax.get_xticklabels(), fontsize=16)  # increase font size
         plt.tick_params(axis='x', which='major', labelsize=14)
@@ -1445,9 +1446,9 @@ def plot_Bks_ASDs( Bkdata, delt=86400, plotpsds=True, plotfscan=False, removeout
             yl.append('0')
           else:
             if item < 0:
-              yl.append(r'$-\nicefrac{1}{%d}$' % (-1./item))
+              yl.append(r'$-\sfrac{1}{%d}$' % (-1./item))
             else:
-              yl.append(r'$\nicefrac{1}{%d}$' % (1./item))
+              yl.append(r'$\sfrac{1}{%d}$' % (1./item))
         ax.set_yticklabels(yl)
         plt.tick_params(axis='y', which='major', labelsize=14)
 
@@ -1958,6 +1959,25 @@ def antenna_response( gpsTime, ra, dec, psi, det ):
   detval = lal.CachedDetectors[detector]
   response = detval.response
 
+  # check if ra and dec are floats or strings (if strings in hh/dd:mm:ss.s format then convert to rads)
+  if not isinstance(ra, float):
+    if isinstance(ra, str):
+      if len(ra.split(':')) == 3:
+        ra = ra_to_rad(ra)
+      else:
+        raise ValueError, "Right ascension string '%s' not formatted properly" % ra
+    else:
+      raise ValueError, "Right ascension must be a 'float' in radians or a string of the format 'hh:mm:ss.s'"
+
+  if not isinstance(dec, float):
+    if isinstance(dec, str):
+      if len(dec.split(':')) == 3:
+        dec = dec_to_rad(dec)
+      else:
+        raise ValueError, "Declination string '%s' not formatted properly" % ra
+    else:
+      raise ValueError, "Declination must be a 'float' in radians or a string of the format 'dd:mm:ss.s'"
+
   # check if gpsTime is just a float or int, and if so convert into an array
   if isinstance(gpsTime, float) or isinstance(gpsTime, int):
     gpsTime = np.array([gpsTime])
@@ -2197,7 +2217,7 @@ def pulsar_mcmc_to_posterior(chainfiles):
       mcmcChain = read_pulsar_mcmc_file(cfile)
 
       # if no chain found then exit with None's
-      if mcmcChain == None:
+      if mcmcChain is None:
         return None, None, None, None
 
       # find number of effective samples for the chain
@@ -2212,7 +2232,7 @@ def pulsar_mcmc_to_posterior(chainfiles):
       neffs.append(math.floor(np.mean(neffstmp)))
 
       #nskip = math.ceil(mcmcChain.shape[0]/min(neffstmp))
-      nskip = math.ceil(mcmcChain.shape[0]/np.mean(neffstmp))
+      nskip = int(math.ceil(mcmcChain.shape[0]/np.mean(neffstmp)))
 
       # output every nskip (independent) value
       mcmc.append(mcmcChain[::nskip,:])
@@ -2355,44 +2375,102 @@ def read_pulsar_mcmc_file(cf):
   return cfdata
 
 
-# Function to convert nested sample files output from pulsar_parameter_estimation_nested, and already
-# parsed through lalapps_nest2pos into a posterior object. The log(evidence ratio) (signal versus Gaussian
-# noise) is also returned.
-#
-# The inputs are a list of nested sample files and the number of live points used to produce them.
-#
-# Any non-varying parameters in the files are removed from the posterior object. iota is converted
-# back into cos(iota), and if only C22 is varying then it is converted back into h0, and phi22
-# is converted back into phi0.
-def pulsar_nest_to_posterior(nestfile):
+def pulsar_nest_to_posterior(postfile, nestedsamples=False, removeuntrig=True):
+  """
+  This function will import a posterior sample file created by `lalapps_nest2pos` (or a nested
+  sample file). It will be returned as a Posterior class object from `bayespputils`. The
+  signal evidence and noise evidence are also returned.
+
+  Parameters
+  ----------
+  postfile : str, required
+      The file name of the posterior or nested sample file. In general this should be a HDF5 file with the extension
+      '.hdf' or '.h5', although older format ascii files are still supported at the moment.
+  nestedsamples : bool, default: False
+      If the file being input contains nested samples, rather than posterior samples, then this flag must be set to
+      True
+  removeuntrig : bool, default: True
+      If this is True then any parameters that are sines or cosines of a value will have the value removed if present
+      e.g. if cosiota and iota exist then iota will be removed.
+  """
+
   from pylal import bayespputils as bppu
+  from pylal.bayespputils import replace_column
+
+  fe = os.path.splitext(postfile)[-1].lower() # file extension
 
   # combine multiple nested sample files for an IFO into a single posterior (copied from lalapps_nest2pos)
-  peparser = bppu.PEOutputParser('common')
+  if fe == '.h5' or fe == '.hdf': # HDF5 file
+    if nestedsamples:
+      # use functions from lalapps_nest2pos to read values from nested sample files i.e. not a posterior file created by lalapps_nest2pos
+      try:
+        from lalinference.io import read_samples
+        import lalinference
 
-  if '.gz' in nestfile:
+        samples = read_samples(postfile, tablename=lalinference.LALInferenceHDF5NestedSamplesDatasetName)
+        params = samples.colnames
+
+        # make everything a float, since that's what's excected of a CommonResultsObj
+        for param in params:
+          replace_column(samples, param, samples[param].astype(float))
+
+        nsResultsObject = (samples.colnames, samples.as_array().view(float).reshape(-1, len(params)))
+      except:
+        raise IOError, "Could not import nested samples"
+    else: # assume posterior file has been created with lalapps_nest2pos
+      peparser = bppu.PEOutputParser('hdf5')
+      nsResultsObject = peparser.parse(postfile)
+  elif fe == '.gz': # gzipped file
     import gzip
-    nsResultsObject = peparser.parse(gzip.open(nestfile, 'r'))
-  else:
-    nsResultsObject = peparser.parse(open(nestfile, 'r'))
+    peparser = bppu.PEOutputParser('common')
+    nsResultsObject = peparser.parse(gzip.open(postfile, 'r'))
+  else: # assume an ascii text file
+    peparser = bppu.PEOutputParser('common')
+    nsResultsObject = peparser.parse(open(postfile, 'r'))
 
   pos = bppu.Posterior( nsResultsObject, SimInspiralTableEntry=None, votfile=None )
 
-  # convert iota back to cos(iota)
-  # create 1D posterior class of cos(iota) values
-  cipos = None
-  cipos = bppu.PosteriorOneDPDF('cosiota', np.cos(pos['iota'].samples))
-
-  # add it back to posterior and remove iota samples
-  pos.append(cipos)
-  pos.pop('iota')
-
-  # remove any unchanging variables
+  # remove any unchanging variables and randomly shuffle the rest
   pnames = pos.names
+  nsamps = len(pos[pnames[0]].samples)
+  permarr = np.arange(nsamps)
+  np.random.shuffle(permarr)
   for pname in pnames:
     # check first and last samples are the same
     if pos[pname].samples[0] - pos[pname].samples[-1] == 0.:
       pos.pop(pname)
+    else:
+      # shuffle
+      shufpos = None
+      shufpos = bppu.PosteriorOneDPDF(pname, pos[pname].samples[permarr])
+      pos.pop(pname)
+      pos.append(shufpos)
+
+  # check whether iota has been used
+  try:
+    posIota = pos['iota'].samples
+  except:
+    posIota = None
+
+  if posIota is not None:
+    cipos = None
+    cipos = bppu.PosteriorOneDPDF('cosiota', np.cos(posIota))
+    pos.append(cipos)
+    if removeuntrig:
+      pos.pop('iota')
+
+  # check whether sin(i) binary parameter has been used
+  try:
+    posI = pos['i'].samples
+  except:
+    posI = None
+
+  if posI is not None:
+    sinipos = None
+    sinipos = bppu.PosteriorOneDPDF('sini', np.sin(posI))
+    pos.append(sinipos)
+    if removeuntrig:
+      pos.pop('i')
 
   # convert C22 back into h0, and phi22 back into phi0 if required
   try:
@@ -2425,11 +2503,21 @@ def pulsar_nest_to_posterior(nestfile):
       pos.append(phi0pos)
       pos.pop('phi22')
 
-  # get evidence (as in lalapps_nest2pos) (assume evidence in files suffixed '_B.txt')
-  B = np.loadtxt(nestfile.replace('.gz', '')+'_B.txt')
+  # get evidence (as in lalapps_nest2pos)
+  if fe == '.h5' or fe == '.hdf':
+    # read evidence from HDF5 file
+    hdf = h5py.File(postfile, 'r')
+    a = hdf['lalinference']['lalinference_nest']
+    sigev = a.attrs['log_evidence']
+    noiseev = a.attrs['log_noise_evidence']
+    hdf.close()
+  else:
+    B = np.loadtxt(postfile.replace('.gz', '')+'_B.txt')
+    sigev = B[1]
+    noiseev = B[2]
 
   # return posterior samples, signal evidence (B[1]) and noise evidence (B[2])
-  return pos, B[1], B[2]
+  return pos, sigev, noiseev
 
 
 # function to add two exponentiated log values and return the log of the result
@@ -2532,6 +2620,115 @@ def get_chunk_lengths( ts, chunkMax ):
   return chunkLengths
 
 
+def pulsar_estimate_snr(source, det, tstart, duration, Sn, dt=1800):
+  """
+  A function to estimate the signal-to-noise ratio of a pulsar signal (for a triaxial neutron
+  star emitting at twice the rotation frequency from the l=m=2 quadrupole mode) in a given
+  detector over a given time range, for a given one-sided power spectral density.
+
+  Inputs
+  ------
+      source - a dictionary containing 'h0', 'cosiota', 'psi', 'ra' (rads or string), 'dec' (rads or string)
+         det - the detector, e.g. 'H1'
+      tstart - a GPS start time
+    duration - a signal duration (seconds)
+          Sn - a one-sided power spectral density
+          dt - timestep for antenna response calculation [default: 1800s]
+
+  Returns
+  -------
+  rho        - an estimate of the optimal matched filter SNR
+  """
+
+  # if duration is greater than a sidereal day then just calculate the antenna pattern for 1 sidereal day plus the remainder
+  sidday = 86164.0905 # one sidereal day in seconds
+  Ndays = duration/sidday
+  Nsamps = np.floor(sidday/dt) # number of time samples in 1 sidereal day
+  tend = tstart + duration
+
+  sFp = 0
+  sFc = 0
+
+  # get antenna pattern and sum over the square of it
+  if Ndays > 1:
+    tts = np.arange(tstart, tstart+sidday, dt)
+
+    # get antenna pattern over one sidereal day
+    Fp, Fc = antenna_response(tts, source['ra'], source['dec'], source['psi'], det )
+
+    sFp = np.floor(Ndays)*np.sum(Fp**2)
+    sFc = np.floor(Ndays)*np.sum(Fc**2)
+
+  # add on extra fractional part of a day
+  if np.floor(Ndays)*sidday > dt:
+    tts = np.arange(tstart+sidday*np.floor(Ndays), tend, dt)
+
+    Fp, Fc = antenna_response(tts, source['ra'], source['dec'], source['psi'], det)
+
+    sFp += np.sum(Fp**2)
+    sFc += np.sum(Fc**2)
+
+  # get snr squared
+  snr2 = (source['h0']**2/Sn)*dt*(sFp*(0.5*(1+source['cosiota']**2))**2 + sFc*source['cosiota']**2)
+
+  return np.sqrt(snr2)
+
+
+def pulsar_estimate_h0_from_snr(snr, source, det, tstart, duration, Sn, dt=600):
+  """
+  A function to estimate the signal amplitude of a pulsar signal (for a triaxial neutron star emitting at twice
+  the rotation frequency from the l=m=2 quadrupole mode) for a given SNR in a particular detector over
+  a given time range, for a given one-sided power spectral density.
+
+  Inputs
+  ------
+         snr - the optimal matched filter signal-to-noise ratio
+      source - a dictionary containing 'cosiota', 'psi', 'ra' (rads or string), 'dec' (rads or string)
+         det - the detector, e.g. 'H1'
+      tstart - a GPS start time for the signal
+    duration - a signal duration (seconds)
+          Sn - a one-sided power spectral density
+          dt - timestep for antenna response calculation [default: 600s]
+
+  Returns
+  -------
+  h0         - an estimate of signal amplitude required to give the input SNR
+  """
+
+  # if duration is greater than a sidereal day then just calculate the antenna pattern for 1 sidereal day plus the remainder
+  sidday = 86164.0905 # one sidereal day in seconds
+  Ndays = duration/sidday
+  Nsamps = np.floor(sidday/dt) # number of time samples in 1 sidereal day
+  tend = tstart + duration
+
+  sFp = 0
+  sFc = 0
+
+  # get antenna pattern and sum over the square of it
+  if Ndays > 1:
+    tts = np.arange(tstart, tstart+sidday, dt)
+
+    # get antenna pattern over one sidereal day
+    Fp, Fc = antenna_response(tts, source['ra'], source['dec'], source['psi'], det )
+
+    sFp = np.floor(Ndays)*np.sum(Fp**2)
+    sFc = np.floor(Ndays)*np.sum(Fc**2)
+
+  # add on extra fractional part of a day
+  if np.floor(Ndays)*sidday > dt:
+    tts = np.arange(tstart+sidday*np.floor(Ndays), tend, dt)
+
+    Fp, Fc = antenna_response(tts, source['ra'], source['dec'], source['psi'], det)
+
+    sFp += np.sum(Fp**2)
+    sFc += np.sum(Fc**2)
+
+  # get h0 squared
+  h02 = (snr**2*Sn)/(dt*(sFp*(0.5*(1+source['cosiota']**2))**2 + sFc*source['cosiota']**2))
+
+  return np.sqrt(h02)
+
+
 def pulsar_posterior_grid(dets, ts, data, ra, dec, sigmas=None, paramranges={}, datachunks=30, chunkmin=5,
                           ngrid=50):
   """
