@@ -41,8 +41,8 @@
 /* Funny thing is: __GLIBC__ is not yet defined here.
    If you move features.h below where __GLIBC_ is defined, the the Linux build breaks.
    This seems a problem specific to gcc-4.4 (headers), though, so the workaround is
-   limited to that version. */
-#if defined (__linux__) && defined (__GNUC__) && (__GNUC__ == 4) && (__GNUC_MINOR__ == 4)
+   limited to that version. Addendum: still a problem in gcc-4.8 on Ubuntu 6.06 */
+#if defined (__linux__) && defined (__GNUC__) && (__GNUC__ == 4)
 #include <features.h>
 #ifdef __GLIBC__
 #ifndef __USE_BSD
@@ -140,7 +140,9 @@ extern int boinc_resolve_filename(const char*, char*, int len);
 #define MIN(a,b) ((a)<(b)?(a):(b))
 #endif
 
-#define DEBUG_COMMAND_LINE_MANGLING 1
+#ifndef DEBUG_COMMAND_LINE_MANGLING
+#define DEBUG_COMMAND_LINE_MANGLING 0
+#endif
 
 typedef enum gdbcmd { gdb_dump_core, gdb_attach } gdb_cmd;
 
@@ -850,6 +852,7 @@ static void worker (void) {
   char wu_result_file[MAX_PATH_LEN];
 
   int second_outfile = 0;        /**< flag: is there a second output file, i.e. --SortToplist=3 */
+  int bsgl_outfiles = 0;
   int resultfile_present = 0;
 
   resultfile[0] = '\0';
@@ -1097,6 +1100,13 @@ static void worker (void) {
       second_outfile = -1;
     }
 
+    /* record if there will be a second output file */
+    else if (!strcmp("--SortToplist=6",argv[arg])) {
+      rargv[rarg] = argv[arg];
+      LogPrintf(LOG_DEBUG,"BSGL output files\n");
+      bsgl_outfiles = -1;
+    }
+
     /* set the "flops estimation" */
     else if (MATCH_START("--WUfpops=",argv[arg],l)) {
       estimated_flops = atof(argv[arg]+l);
@@ -1313,10 +1323,16 @@ static void worker (void) {
       if (bundle_size) {
 	char buf[20];
 	unsigned int rlen = strlen(resultfile);
+        long n;
 	strcpy(wu_result_file, resultfile);
-	strcpy(&wu_result_file[rlen-1], myltoa(second_outfile ? current_config_file * 2 : current_config_file, buf, 20));
+	if (second_outfile)
+	  n = current_config_file * 2;
+	else if (bsgl_outfiles)
+	  n = current_config_file * 3;
+	else
+	  n = current_config_file;
+	strcpy(&wu_result_file[rlen-1], myltoa(n, buf, 20));
 	*config_file_arg = config_files[current_config_file];
-
 	if (fp = boinc_fopen(wu_result_file,"r")) {
 	  fclose(fp);
 	  LogPrintf (LOG_NORMAL, "WARNING: Resultfile '%s' present - skipping subWU#%d\n", wu_result_file, current_config_file);
@@ -1371,6 +1387,34 @@ static void worker (void) {
 	}
       }
 
+      /* if there is a file <wuname>_<instance>_0-LV, rename it to <wuname>_<instance>_1 */
+      if (bsgl_outfiles) {
+	unsigned int len = strlen(resultfile);
+	char*lv_file = (char*)malloc(len+12);
+	if (lv_file) {
+	  LogPrintf(LOG_DEBUG,"resultfile '%s' (len %d), current config file: %d\n", resultfile, len, current_config_file);
+	  strcpy(lv_file, resultfile);
+	  strcat(lv_file, "-BSGLtL");
+	  if (boinc_file_exists(lv_file) && resultfile[len-1]=='0') {
+	    resultfile[len-1]='1';
+	    // myltoa(current_config_file*3+1, &resultfile[len-1], 5);
+	    LogPrintf(LOG_DEBUG,"renaming '%s' to '%s'\n", lv_file, resultfile);
+	    boinc_rename(lv_file,resultfile);
+	  }
+	  lv_file[len]='\0';
+	  strcat(lv_file, "-BtSGLtL");
+	  if (boinc_file_exists(lv_file) && resultfile[len-1]=='1') {
+	    resultfile[len-1]='2';
+	    // myltoa(current_config_file*3+2, &resultfile[len-1], 5);
+	    LogPrintf(LOG_DEBUG,"renaming '%s' to '%s'\n", lv_file, resultfile);
+	    boinc_rename(lv_file,resultfile);
+	  }
+	} else {
+	  LogPrintf(LOG_CRITICAL,"ERROR: out of memory, can't allocate lv_file\n");
+	  res = HS_BOINC_EXIT_MEM;
+	}
+      }
+
       current_config_file ++;
     } while (!res && bundle_size && current_config_file < bundle_size);
 
@@ -1689,7 +1733,7 @@ int main(int argc, char**argv) {
 /**
  * sets a checkpoint.
  */
-int write_boinc_gct_checkpoint(const char*filename, toplist_t*tl, toplist_t*t2, UINT4 counter, BOOLEAN do_sync) {
+int write_boinc_gct_checkpoint(const char*filename, toplist_t*tl, toplist_t*t2, toplist_t*t3,UINT4 counter, BOOLEAN do_sync) {
   int ret;
   /* make sure the exception mask isn't messed up by a badly written device driver etc.,
      so restore it periodically */
@@ -1699,7 +1743,7 @@ int write_boinc_gct_checkpoint(const char*filename, toplist_t*tl, toplist_t*t2,
   if (!(boinc_is_standalone() || boinc_time_to_checkpoint()))
     return 1; /* >0, no checkpoint written, no error */
 #endif
-  ret = write_gct_checkpoint(filename, tl, t2, counter, do_sync);
+  ret = write_gct_checkpoint(filename, tl, t2, t3, counter, do_sync);
   fprintf(stderr,"c\n");
   boinc_checkpoint_completed();
   return(ret);
