@@ -94,10 +94,9 @@ typedef struct
 // ----- User variables
 typedef struct
 {
-  /* output */
+  /* SFT output */
   CHAR *outSFTdir;		/**< Output directory for SFTs */
   CHAR *outLabel;		/**< 'misc' entry in SFT-filenames, and description entry of output frame filenames */
-  CHAR *outFrameDir;		/**< directory for writing output timeseries in frame files */
   BOOLEAN outSingleSFT;	        /**< use to output a single concatenated SFT */
 
   CHAR *TDDfile;		/**< Filename for ASCII output time-series */
@@ -123,8 +122,11 @@ typedef struct
 
   CHAR *noiseSFTs;		/**< Glob-like pattern specifying noise-SFTs to be added to signal */
 
+  /* Frame input and output */
+  CHAR *outFrameDir;		/**< directory for writing output timeseries in frame files */
   LALStringVector *inFrames;	/**< frame glob-patterns or cache files of time-series data to be added to output (one per IFO) */
   LALStringVector *inFrChannels;/**< list of frame channels to read time-series data from (one per IFO) */
+  LALStringVector *outFrChannels;/**< list of output frame channel names to write (one per IFO) */
 
   /* Window function [OPTIONAL] */
   CHAR *SFTWindowType;		/**< Windowing function to apply to the SFT time series */
@@ -182,7 +184,7 @@ main(int argc, char *argv[])
 
   PulsarParamsVector *injectionSources = NULL;
   if ( uvar.injectionSources ) {
-    XLAL_CHECK ( (injectionSources = XLALPulsarParamsFromUserInput ( uvar.injectionSources ) ) != NULL, XLAL_EFUNC );
+    XLAL_CHECK ( (injectionSources = XLALPulsarParamsFromUserInput ( uvar.injectionSources, NULL ) ) != NULL, XLAL_EFUNC );
   }
 
   CWMFDataParams XLAL_INIT_DECL(DataParams);
@@ -279,6 +281,10 @@ main(int argc, char *argv[])
       char *fname;
 
       char *hist = XLALUserVarGetLog (UVAR_LOGFMT_CMDLINE);
+      if ( XLALUserVarWasSet ( &uvar.outFrChannels ) ) {
+        XLAL_CHECK ( uvar.outFrChannels->length == mTseries->length, XLAL_EINVAL, "--outFrChannels: number of channel names (%d) must agree with number of IFOs (%d)\n",
+                     uvar.outFrChannels->length, mTseries->length );
+      }
 
       for ( UINT4 X=0; X < mTseries->length; X ++ )
         {
@@ -299,8 +305,19 @@ main(int argc, char *argv[])
 
           /* add timeseries to the frame - make sure to change the timeseries name since this is used as the channel name */
           char buffer[LALNameLength];
-          written = snprintf ( buffer, LALNameLength, "%s:%s", Tseries->name, uvar.outLabel );
-          XLAL_CHECK ( written < LALNameLength, XLAL_ESIZE, "Updated frame name exceeds max length (%d): '%s'\n", LALNameLength, buffer );
+          // if output frame channel names given, use those
+          if ( XLALUserVarWasSet ( &uvar.outFrChannels ) ) {
+            written = snprintf ( buffer, sizeof(buffer), "%s", uvar.outFrChannels->data[X] );
+            if ( buffer[2] == ':' ) { // check we got correct IFO association
+              XLAL_CHECK ( (buffer[0] == Tseries->name[0]) && (buffer[1] == Tseries->name[1]), XLAL_EINVAL,
+                           "Possible IFO mismatch: outFrChannel[%d] = '%s', IFO = '%c%c': be careful about --outFrChannel ordering\n", X, buffer, Tseries->name[0], Tseries->name[1] );
+            } // if buffer[2]==':'
+          } else if ( XLALUserVarWasSet ( &uvar.inFrChannels ) ) { // otherwise: if input frame channel names given, use them for output, append "-<outLabel>"
+            written = snprintf ( buffer, sizeof(buffer), "%s-%s", uvar.inFrChannels->data[X], uvar.outLabel );
+          } else { // otherwise: fall back to <IFO>:<outLabel> channel name
+            written = snprintf ( buffer, sizeof(buffer), "%c%c:%s", Tseries->name[0], Tseries->name[1], uvar.outLabel );
+          }
+          XLAL_CHECK ( written < LALNameLength, XLAL_ESIZE, "Output frame name exceeded max length (%d): '%s'\n", LALNameLength, buffer );
           strcpy ( Tseries->name, buffer );
 
           XLAL_CHECK ( (XLALFrameAddREAL8TimeSeriesProcData ( outFrame, Tseries ) == XLAL_SUCCESS ) , XLAL_EFUNC );
@@ -474,7 +491,6 @@ XLALInitMakefakedata ( ConfigVars_t *cfg, UserVariables_t *uvar )
         }
       for ( UINT4 X = 0; X < numDetectors; X ++ )
         {
-          const LIGOTimeGPSVector *timestampsX = cfg->multiTimestamps->data[X];
           LALCache *cache;
           XLAL_CHECK ( (cache = XLALCacheImport ( uvar->inFrames->data[X] )) != NULL, XLAL_EFUNC, "Failed to import cache file '%s'\n", uvar->inFrames->data[X] );
           // this is a sorted cache, so extract its time-range:
@@ -484,8 +500,9 @@ XLALInitMakefakedata ( ConfigVars_t *cfg, UserVariables_t *uvar )
           LIGOTimeGPS ts_start;
           REAL8 ts_duration;
           // check that it's consistent with timestamps, if given, otherwise create timestamps from this
-          if ( timestampsX != NULL )	// FIXME: implicitly assumes timestamps are sorted, which is not guaranteed by timestamps-reading from file
+          if ( cfg->multiTimestamps->data[X] != NULL )	// FIXME: implicitly assumes timestamps are sorted, which is not guaranteed by timestamps-reading from file
             {
+              const LIGOTimeGPSVector *timestampsX = cfg->multiTimestamps->data[X];
               REAL8 tStart = XLALGPSGetREAL8( &timestampsX->data[0] );
               REAL8 tEnd   = XLALGPSGetREAL8( &timestampsX->data[timestampsX->length-1]) + timestampsX->deltaT;
               XLAL_CHECK ( tStart >= cache_tStart && tEnd <= cache_tEnd, XLAL_EINVAL, "Detector X=%d: Requested timestamps-range [%.0f, %.0f]s outside of cache range [%.0f,%.0f]s\n",
@@ -497,7 +514,7 @@ XLALInitMakefakedata ( ConfigVars_t *cfg, UserVariables_t *uvar )
             {
               XLALGPSSetREAL8 ( &ts_start, (REAL8)cache_tStart + 1); // cache times can apparently be by rounded up or down by 1s, so shift by 1s to be safe
               ts_duration = cache_duration - 1;
-              XLAL_CHECK ( (timestampsX = XLALMakeTimestamps ( ts_start, ts_duration, uvar->Tsft, uvar->SFToverlap ) ) != NULL, XLAL_EFUNC );
+              XLAL_CHECK ( (cfg->multiTimestamps->data[X] = XLALMakeTimestamps ( ts_start, ts_duration, uvar->Tsft, uvar->SFToverlap ) ) != NULL, XLAL_EFUNC );
             }
           // ----- now open frame stream and read *all* the data within this time-range [FIXME] ----------
           LALFrStream *stream;
@@ -555,10 +572,6 @@ XLALInitUserVars ( UserVariables_t *uvar, int argc, char *argv[] )
   XLALRegisterUvarMember(   outSingleSFT,       BOOLEAN, 's', OPTIONAL, "Write a single concatenated SFT file instead of individual files" );
   XLALRegisterUvarMember( outSFTdir,          STRING, 'n', OPTIONAL, "Output SFTs:  directory for output SFTs");
   XLALRegisterUvarMember(  outLabel,	         STRING, 0, OPTIONAL, "'misc' entry in SFT-filenames or 'description' entry of frame filenames" );
-#ifdef HAVE_LIBLALFRAME
-  XLALRegisterUvarMember ( outFrameDir,	STRING, 'F', OPTIONAL,      "Output Frames: directory for output timeseries frame files");
-#endif
-
   XLALRegisterUvarMember( TDDfile,            STRING, 't', OPTIONAL, "Filename to output time-series into");
 
   XLALRegisterUvarMember( logfile,            STRING, 'l', OPTIONAL, "Filename for log-output");
@@ -591,9 +604,17 @@ XLALInitUserVars ( UserVariables_t *uvar, int argc, char *argv[] )
   /* noise */
   XLALRegisterUvarMember( noiseSFTs,          STRING, 'D', OPTIONAL, "Noise-SFTs to be added to signal (Used also to set IFOs and timestamps)");
 
+  /* frame input/output options */
 #ifdef HAVE_LIBLALFRAME
-  XLALRegisterUvarMember ( inFrames, 	 STRINGVector,'C', OPTIONAL,  "CSV list (one per IFO) of input frame cache files");
-  XLALRegisterUvarMember ( inFrChannels, STRINGVector,'N', OPTIONAL,  "CSV list (one per IFO) of frame channels to read timeseries from");
+  XLALRegisterUvarMember ( outFrameDir,	  STRING,      'F', OPTIONAL,  "Output Frames: directory for output timeseries frame files");
+  XLALRegisterUvarMember ( inFrames, 	  STRINGVector,'C', OPTIONAL,  "CSV list (one per IFO) of input frame cache files");
+  XLALRegisterUvarMember ( inFrChannels,  STRINGVector,'N', OPTIONAL,  "CSV list (one per IFO) of frame channels to read timeseries from");
+  XLALRegisterUvarMember ( outFrChannels, STRINGVector, 0,  NODEFAULT, "CSV list (one per IFO) of output frame channel names [default: <inFrChannels>-<outLabel> or <IFO>:<outLabel>]");
+#else
+  XLALRegisterUvarMember ( outFrameDir,	 STRING,       'F', DEFUNCT, "Need to compile with lalframe support for this option to work");
+  XLALRegisterUvarMember ( inFrames,     STRINGVector, 'C', DEFUNCT, "Need to compile with lalframe support for this option to work");
+  XLALRegisterUvarMember ( inFrChannels, STRINGVector, 'N', DEFUNCT, "Need to compile with lalframe support for this option to work");
+  XLALRegisterUvarMember ( outFrChannels, STRINGVector, 0,  DEFUNCT, "Need to compile with lalframe support for this option to work");
 #endif
 
   XLALRegisterUvarMember(  version,             BOOLEAN, 'V', SPECIAL, "Output version information");
@@ -603,16 +624,12 @@ XLALInitUserVars ( UserVariables_t *uvar, int argc, char *argv[] )
 
   // ----- deprecated but still supported options [throw warning if used] (only shown in help at lalDebugLevel >= info) ----------
 #ifdef HAVE_LIBLALFRAME
-  XLALRegisterUvarMember ( TDDframedir,	STRING,  0, DEPRECATED, "Use --outFrameDir instead");
+  XLALRegisterUvarMember ( TDDframedir,	STRING,  0,  DEPRECATED, "Use --outFrameDir instead");
+#else
+  XLALRegisterUvarMember ( TDDframedir,	 STRING, 0,  DEFUNCT,   "Need to compile with lalframe support. BUT this option is deprecated and --outFrameDir should be used instead");
 #endif
-
   // ----- obsolete and unsupported options [throw error if used] (never shown in help) ----------
-#if !defined(HAVE_LIBLALFRAME)
-  XLALRegisterUvarMember ( outFrameDir,	 STRING,       'F', DEFUNCT, "Need to compile with lalframe support for this option to work");
-  XLALRegisterUvarMember ( TDDframedir,	 STRING,        0,  DEFUNCT, "Need to compile with lalframe support for this option to work");
-  XLALRegisterUvarMember ( inFrames,     STRINGVector, 'C', DEFUNCT, "Need to compile with lalframe support for this option to work");
-  XLALRegisterUvarMember ( inFrChannels, STRINGVector, 'N', DEFUNCT, "Need to compile with lalframe support for this option to work");
-#endif
+
 
   /* read cmdline & cfgfile  */
   BOOLEAN should_exit = 0;
